import java.io.*;
import java.util.Properties;
import javax.mail.internet.MimeUtility;
import javax.mail.MessagingException;
import org.w3c.dom.*;
import com.lowagie.text.DocumentException;
import org.xhtmlrenderer.pdf.ITextRenderer;
import org.xhtmlrenderer.pdf.ITextFontResolver;
import org.xhtmlrenderer.util.XRRuntimeException;
import org.xhtmlrenderer.render.Box;
import org.xhtmlrenderer.resource.XMLResource;
import org.xhtmlrenderer.css.constants.CSSName;
import org.xml.sax.InputSource;

public class PdfConverter {

    private Properties fontMap = new Properties();

    public PdfConverter() throws IOException {
        System.setProperty("java.awt.headless", "true");

        //Read font mapping specification.
        String file = System.getenv("EPAGES_CONFIG") + "/PdfFontMapping.conf";
        try {
            InputStream fontMapConf = new FileInputStream(new File(file));
            fontMap.load(fontMapConf);
        } catch(FileNotFoundException fnfe) {
            Log("File not found: " + file);
        }
    }

    public String Convert(final String htmlCode)
                    throws MessagingException, IOException, DocumentException {

        ByteArrayOutputStream bout = new ByteArrayOutputStream();
        ITextRenderer renderer = new ITextRenderer();

        //Get true type fonts from font directory and add them to the renderer.
        String fontDir = System.getenv("EPAGES_CARTRIDGES") + "/DE_EPAGES/Core/Data/Fonts";
        File dir = new File(fontDir);
        if(dir.exists()){
            String[] fontList = dir.list();
            ITextFontResolver resolver = renderer.getFontResolver();
            for (int i = 0; i < fontList.length; i++) {
                //Add fonts using the Unicode encoding with horizontal writing.
                //http://api.itextpdf.com/com/itextpdf/text/pdf/BaseFont.html
                String filePath = fontDir + "/" + fontList[i];
                String lcp = filePath.toLowerCase();
                if(lcp.endsWith(".ttf") || lcp.toLowerCase().endsWith(".otf") || lcp.endsWith(".ttc")){
                    resolver.addFont(filePath, "Identity-H", true);
                }
            }
        } else {
            Log("Directory not found: " + dir);
        }

        PrintStream oldErr = System.err;
        ByteArrayOutputStream SystemError = new ByteArrayOutputStream();

        //Generate pdf output from html string
        try {

            //Redirect System.Err output stream
            System.setErr(new PrintStream(SystemError, true));

            //Load xml resource
            InputSource is = new InputSource(new BufferedReader(new StringReader(htmlCode)));
            Document dom = XMLResource.load(is).getDocument();

            //Process font mapping
            if(fontMap.size() != 0){
                renderer.setDocument(dom,null);
                renderer.layout();

                //for "style" attributes
                mapXmlFonts(dom.getDocumentElement());

                //for "class" attributes
                mapBoxFonts(renderer.getRootBox());
            }

            //Create Pdf
            renderer.setDocument(dom,null);
            renderer.layout();
            renderer.createPDF(bout);
        } catch (XRRuntimeException ex){
            throw new XRRuntimeException(SystemError.toString());
        } finally {
            //Restore System.Err
            System.setErr(oldErr);

        }

        //Convert Pdf output to base64 string
        byte[] b = bout.toByteArray();
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        OutputStream b64os = MimeUtility.encode(baos, "base64");
        b64os.write(b);
        b64os.close();
        return new String(baos.toByteArray());
    }

    //This function looks for dom elements with "style" attributes
    //and replaces all font families with their substitutes.
    private void mapXmlFonts(Element e){

        //Handle "style" attributes.
        if(e.hasAttribute("style")){
            String style = e.getAttribute("style");
            String[] fontFamilies = style.split(":")[1].replaceAll(";","").split(",");
            e.setAttribute("style", matchFontFamilies(style, fontFamilies));
        }

        //Process all children recursively.
        NodeList childNodes = e.getChildNodes();
        for(int i = 0; i < childNodes.getLength(); i++){
            Node eChild = childNodes.item(i);
            if(eChild.getNodeType() == Node.ELEMENT_NODE){
                mapXmlFonts((Element)eChild);
            }
        }
    }

    //This function looks for dom elements with "class" attributes.
    //It determines the font families contained in it's style, replaces
    //all font families with their substitutes and sets the new style.
    private void mapBoxFonts(Box boxObj){

        //Look for DOM elements and handle "class" attributes.
        Element e = boxObj.getElement();
        if(e != null && e.hasAttribute("class")){
            String style = boxObj.getStyle().valueByName(CSSName.FONT_FAMILY).toString();
            String[] fontFamilies = boxObj.getStyle().valueByName(CSSName.FONT_FAMILY).asStringArray();
            e.setAttribute("style", "font-family: " + matchFontFamilies(style, fontFamilies));
        }

        //Process all children recursively.
        for(int i = 0; i < boxObj.getChildCount(); i++){
            Box boxChild = boxObj.getChild(i);
            mapBoxFonts((Box)boxChild);
        }
    }

    private String matchFontFamilies(String style, String[] fontFamilies){
        for(int h= 0; h < fontFamilies.length; h++ ){
            String fontName = fontFamilies[h].trim();
            String substitute = fontMap.getProperty(fontName);
            if(substitute != null){
                style = style.replaceAll(fontName, substitute);
            }
        }
        return style;
    }

    private void Log(String text) throws IOException {
        String file = System.getenv("EPAGES_LOG") + "/error.log";
        BufferedWriter out = new BufferedWriter(new FileWriter(file,true));
        out.write(text + "\n");
        out.close();
    }

}
