#!perl -w
use Test::More;
use strict;
use Config;
use File::Temp qw( tempdir );

use DE_EPAGES::Core::API::File qw ( AbsolutePath WriteFile CreateDirectory DeleteDirectory );
use DE_EPAGES::Database::API::Config qw( StoreConnections StoreTypes );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Warning qw( BlockWarnings );

my %Configuration = (
    ASPoolDB => <<'END_FILE',
[Connection]
active=1
datasource=dbi:SQLite:dbname=_EPAGES_CONFIG_/ASPool.db
StoreType=ASPoolDB
END_FILE
    Backup => <<'END_FILE',
[Connection]
active=1
datasource=dbi:mysql
Database=mysql
login=root
passwd=epages
END_FILE
    EbayCategories => <<'END_FILE',
[Connection]
active=1
datasource=dbi:SQLite:dbname=_EPAGES_CONFIG_/EbayCategories.db
END_FILE
    Site => <<'END_FILE',
[Connection]
active=1
datasource=dbi:mysql
StoreType=Site
Database=sitedb
login=usr_sitedb
passwd=epages
END_FILE
    Store => <<'END_FILE',
[Connection]
active=1
datasource=dbi:mysql
StoreType=Store
Database=storedb
login=usr_storedb
passwd=epages
END_FILE
    );

sub Main {

#    return if $Config{'osname'} eq 'MSWin32';

    my $PerlScript = AbsolutePath($0);

    if( $Config{'osname'} ne 'MSWin32' ) {
        if ((getpwuid($>))[0] eq 'root') {
            my $Command = "su - ep_appl -c \"$ENV{PERL} $PerlScript\"";
            print $Command, "\n";
            system("$Command");
            return;
        }
    }

    plan tests => 6;

    my $ConfigDir = tempdir();
    my $DBDir ="$ConfigDir/Database.d";

    CreateDirectory($DBDir, { recursive => 1 });

    local $ENV{EPAGES_CONFIG} = $ConfigDir;

    foreach my $Store (keys %Configuration) {
        WriteFile( "$DBDir/$Store.conf", \$Configuration{$Store} );
    }

    my (@Expected, @Found);

    foreach my $Store (sort keys %Configuration) {
        next unless $Configuration{$Store} =~ /StoreType/s;
        push @Expected, $Store;
    }
    @Found = sort @{StoreTypes()};

    # test 1: did we found all stores with StoreType?
    ok(@Found ~~ @Expected, "(@Found) ~~ (@Expected)");

    @Expected = (sort keys %Configuration);
    @Found = sort @{StoreConnections()};

    # test 2: did we found all store connections?
    ok(@Found ~~ @Expected, "(@Found) ~~ (@Expected)");

    chmod oct(111), "$DBDir/Site.conf"; ## no critic qw(ProhibitMagicNumber)
    my $aWarnings = BlockWarnings(sub {@Found = sort @{StoreTypes()}});

    SKIP: {
         skip 'No warnings expected on Windows', 2 if $Config{'osname'} eq 'MSWin32';

        # test 3: did we got the expected warning?
        is($aWarnings->[0]->code, 'FileReadFailed', 'warning code');

        # test 4: with the expected error file?
        is($aWarnings->[0]->vars->{FileName}, 'Site.conf', 'warning file name');
    }

    @Expected = ();
    foreach my $Store (sort keys %Configuration) {
        next if $Store eq 'Site' && $Config{'osname'} ne 'MSWin32';
        next unless $Configuration{$Store} =~ /StoreType/s;
        push @Expected, $Store;
    }

    # test 5: did we found all stores but Site with StoreType?
    ok(@Found ~~ @Expected, "(@Found) ~~ (@Expected)");

    @Expected = ();
    foreach my $Store (sort keys %Configuration) {
        next if $Store eq 'Site' && $Config{'osname'} ne 'MSWin32';
        push @Expected, $Store;
    }
    @Found = sort @{StoreConnections()};

    # test 6: did we found all store connections but Site?
    ok(@Found ~~ @Expected, "(@Found) ~~ (@Expected)");

    DeleteDirectory($ConfigDir, { recursive => 1 });
}

RunScript(Sub=>\&Main);
