use Test::More;
use strict;
use DE_EPAGES::Core::API::File qw ( DeleteDirectory ExistsDirectory );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::PerlTools qw ( UsePackage );
use DE_EPAGES::CodeGenerator::API::CreateCode qw ( CreateCode );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore GetCurrentDBHandle );
use DE_EPAGES::Database::API::DatabaseInstaller;

RunScript( Sub => sub {
    local $| = 1;
    Initialize();
    Main();
    Cleanup();
});

sub Main {

    GenerateCode();
    InstallCartridge();
    UseCartridge();
    UninstallCartridge();

    done_testing();
}

sub GenerateCode {

    foreach my $File ( glob '*.pdm' ) {
        ok( _GenerateCodeForModel($File), "generate from $File" );
    }
}

sub _GenerateCodeForModel {
    my ($ModelFile) = @_;

    CreateCode({
        'BUILD_FILE'        => '../Templates/epages/Build.xml',
        'SOURCE_FILE'       => $ModelFile,
        'TARGET_DIR'        => '.',
    });

    return 1;
}

sub InstallCartridge {

    RunOnStore( Store => GetTestSite(), Sub => sub {
        my $Installer = DE_EPAGES::Database::API::DatabaseInstaller->new(
            'CartridgeDirectory' => 'DE_EPAGES/DAL_TEST',
            'Version'            => '1.0',
        );

        # fake install cartridge DE_EPAGES::DAL_TEST
        local $ENV{EPAGES_CARTRIDGES} = '.';
        my $DatabaseDir = $Installer->databaseDirectory;
        ok( $Installer->hasDBSchema, 'hasDBSchema' );
        ok( -d $DatabaseDir, 'Database directory exists' );
        $Installer->processDatabaseDir( $DatabaseDir );
        ok( 1, 'processDatabaseDir' );
    });
}

sub UninstallCartridge {

    RunOnStore( Store => GetTestSite(), Sub => sub {
        my $Installer = DE_EPAGES::Database::API::DatabaseInstaller->new(
            'CartridgeDirectory' => 'DE_EPAGES/DAL_TEST',
            'Version'            => '1.0',
            'Force' => 1,
        );

        # fake uninstall cartridge DE_EPAGES::DAL_TEST
        local $ENV{EPAGES_CARTRIDGES} = '.';
        my $DatabaseDir = $Installer->databaseDirectory;
        $Installer->cleanupDatabaseDir( $DatabaseDir );
    });
}

sub UseCartridge {

    RunOnStore( Store => GetTestSite(), Sub => sub {
        UsePackage('DE_EPAGES::DAL_TEST::DAL::Test01');
        DE_EPAGES::DAL_TEST::DAL::Test01->import(qw( DAL_InsertTable1 DAL_UpdateTable1 DAL_DeleteTable1 DAL_GetInfoTable1 ));
        _TestTable1();

        _TestTable2();
    });
}

# test the driver-independend DAL
sub _TestTable1 {

    my $MasterID = DAL_InsertTable1({
        TestTime => DateTime->new(
            year   => 2003,
            month  => 4,
            day    => 25,
            hour   => 15,
            minute => 53,
            second => 1,
            nanosecond => 500000000,
            time_zone => 'Europe/Brussels',
        ),
        TestDate => DateTime->new(
            year   => 2003,
            month  => 4,
            day    => 25,
            time_zone => 'Europe/Brussels',
        ),
        TestInteger => 1234,
        TestString => 'test string',
        TestFloat => 123.4
    });
    my $hInfo0 = DAL_GetInfoTable1($MasterID);
    is( $hInfo0->{'TestString'}, 'das war der test', 'DAL_GetInfoTable1 TestString' );

    DAL_UpdateTable1($MasterID, {
        TestTime => DateTime->new(
            year   => 2004,
            month  => 4,
            day    => 25,
            hour   => 17,
            minute => 53,
            second => 1,
            nanosecond => 500000000,
            time_zone => 'Europe/Brussels',
        ),
        TestString  => 'das war der test',
    });
    my $hInfo1 = DAL_GetInfoTable1($MasterID);
    is( $hInfo1->{'TestInteger'}, 1234, 'DAL_GetInfoTable1 TestInteger' ); ## no critic qw( ProhibitMagicNumbers )
    is( $hInfo1->{'TestFloat'}, 123.4, 'DAL_GetInfoTable1 TestFloat' ); ## no critic qw( ProhibitMagicNumbers )

    # columns TestTime has the attribute AutoFill=ChangeDate;
    my $delta_ms = $hInfo1->{'TestTime'}->subtract_datetime_absolute( GetCurrentDBHandle()->currentDateTime )->seconds;
    ok( $delta_ms <= 1, 'DAL_GetInfoTable1 TestTime' );

    is( $hInfo1->{'TestDate'}, '2003-04-25T00:00:00', 'DAL_GetInfoTable1 TestDate' );
    is( $hInfo1->{'TestString'}, 'das war der test', 'DAL_GetInfoTable1 TestString' );

    my $ahAllInfo = DAL_GetAllInfoTable1s();
    is( scalar @$ahAllInfo, 1, 'DAL_GetAllInfoTable1s' );

    DAL_DeleteTable1($MasterID);

    $ahAllInfo = DAL_GetAllInfoTable1s();
    is( scalar @$ahAllInfo, 0, 'DAL_DeleteTable1' );
}

# test the driver-specific DAL
sub _TestTable2 {

    my @TABLE2_DATA = (
        {
            'MasterID'  => 1,
            'MasterID2' => 2,
            'MasterID3' => 3,
            'Name'      => 5,
            'Name2'     => 5,
            'Name3'     => 6,
        },{
            'MasterID'  => 1,
            'MasterID2' => 3,
            'MasterID3' => 4,
            'Name'      => 5,
            'Name2'     => 6,
            'Name3'     => 7,
        }
    );

    my $dbi = GetCurrentDBHandle();
    my $driverName = $dbi->driverName;
    my $DAL = "DE_EPAGES::DAL_TEST::DAL::${driverName}::Test02";
    UsePackage( $DAL );
    foreach my $data (@TABLE2_DATA) {
        $DAL->InsertTable2($dbi, $data);
    }
    my $ahInfo = $DAL->GetAllInfoTable2s($dbi);
    is( scalar @$ahInfo, 2, 'GetAllInfoTable2s' );

    $ahInfo = $DAL->GetInfoTable2sByName($dbi, 5);
    is( scalar @$ahInfo, 2, 'GetInfoTable2sByName with 1 param' );

    $ahInfo = $DAL->GetInfoTable2sByName($dbi, 5, 6);
    is( scalar @$ahInfo, 1, 'GetInfoTable2sByName with 2 params' );

    $ahInfo = $DAL->GetInfoTable2sByMasterID($dbi, 1);
    is( scalar @$ahInfo, 2, 'GetInfoTable2sByMasterID with 1 param' );

    $ahInfo = $DAL->GetInfoTable2sByMasterID($dbi, 1, 2);
    is( scalar @$ahInfo, 1, 'GetInfoTable2sByMasterID with 2 params' );

    foreach my $data (@TABLE2_DATA) {
        $DAL->DeleteTable2($dbi, @$data{qw (MasterID MasterID2 MasterID3)});
    }

    $ahInfo = $DAL->GetInfoTable2sByMasterID($dbi, 1);
    is( scalar @$ahInfo, 0, 'DeleteTable2/GetInfoTable2sByMasterID' );

    $ahInfo = $DAL->GetAllInfoTable2s($dbi, 1);
    is( scalar @$ahInfo, 0, 'DeleteTable2/GetAllInfoTable2s' );
}

sub Initialize {
    Cleanup();
}

sub Cleanup {
    if( ExistsDirectory('DE_EPAGES') ) {
        UninstallCartridge();
        DeleteDirectory( 'DE_EPAGES', { 'recursive' => 1 } );
    }
}

1;

