#!perl -w
use Test::More;
use strict;
use Encode qw( decode );
use DE_EPAGES::Core::API::Error qw (ExistsError GetError);
use DE_EPAGES::Core::API::Url qw(
    UrlEncodeValue
    BuildParameterString
    StringIsURL
);

plan tests => 22;

# UrlEncodeValue
# the following characters will NOT be encoded:
#     "-" / "." / "_" / "~"
#     gen-delims  = ":" / "/" / "?" / "#" / "[" / "]" / "@"
# while these reserved characters have to be encoded:
#     sub-delims  = "!" / "$" / "&" / "'" / "(" / ")"
#                 / "*" / "+" / "," / ";" / "="
# UrlEncodeValue re-decodes the slash / ..


eval {
  UrlEncodeValue();
};
ok( ExistsError(), 'Method UrlEncodeValue throws an error if called without an argument' );
my $ErrorCode;
if(ExistsError()) {
  my $Error = GetError();
  $ErrorCode= $Error->code;
}
cmp_ok($ErrorCode, 'eq', 'PARAMETER_MISSING', 'UrlEncodeValue undef input error code is PARAMETER_MISSING');

cmp_ok(UrlEncodeValue(''), 'eq', '', 'UrlEncodeValue empty input');
cmp_ok(UrlEncodeValue('-._~:/?#[]@!$&\'()*+,;="Test//_Str i n G%12 '), 'eq',
    '-._~%3A/%3F%23%5B%5D%40%21%24%26%27%28%29%2A%2B%2C%3B%3D%22Test//_Str+i+n+G%2512+',
    'UrlEncodeValue teststring');
cmp_ok(UrlEncodeValue('-._~:/?#[]@!$&\'()*+,;="Test//_Str i n G%12 Αθηνά', 1), 'eq',
    '-._~%3A/%3F%23%5B%5D%40%21%24%26%27%28%29%2A%2B%2C%3B%3D%22Test//_Str+i+n+G%2512+%CE%91%CE%B8%CE%B7%CE%BD%CE%AC',
    'UrlEncodeValue teststring noutf8');
cmp_ok(UrlEncodeValue('-._~:/?#[]@!$&\'()*+,;="Test//_Str i n G%12 Αθηνά'), 'eq',
    '-._~%3A/%3F%23%5B%5D%40%21%24%26%27%28%29%2A%2B%2C%3B%3D%22Test//_Str+i+n+G%2512+%C3%8E%C2%91%C3%8E%C2%B8%C3%8E%C2%B7%C3%8E%C2%BD%C3%8E%C2%AC',
    'UrlEncodeValue teststring utf8');
cmp_ok(UrlEncodeValue('?pageid=10&lang=en&title=Section%20Title</h1><script>alert(\'XSS%20attack\')</script>'),'eq',
    '%3Fpageid%3D10%26lang%3Den%26title%3DSection%2520Title%3C/h1%3E%3Cscript%3Ealert%28%27XSS%2520attack%27%29%3C/script%3E',
    'UrlEncodeValue script tag');

# BuildParameterString
is( BuildParameterString({
        'ReturnUrl' => 'http://www.shop.de/epages/?ChangeAction=Change&ViewAction=View'
    }),
    'ReturnUrl=http%3A//www.shop.de/epages/%3FChangeAction%3DChange%26ViewAction%3DView',
    'BuildParameterString hash returnurl');

is( BuildParameterString({
        'ChangeAction' => ['BlockMonitor', 'Save'],
        'ViewAction' => 'View'
    }),
    'ChangeAction=BlockMonitor&ChangeAction=Save&ViewAction=View',
    'BuildParameterString hash actions');

is( BuildParameterString({
        'LastName'  => decode("iso-8859-1", "J\x{F6}rgensen"),
        'FirstName' => "Jrg"
    }),
    'FirstName=J%C3%B6rg&LastName=J%C3%B6rgensen',
    'BuildParameterString umlaut, utf8');

is( BuildParameterString({
        'LastName'  => decode("iso-8859-1", "J\x{F6}rgensen"),
        'FirstName' => "Jrg"
    }, 1),
    'FirstName=J%F6rg&LastName=J%F6rgensen',
    'BuildParameterString umlaut, noutf8');

is( BuildParameterString({
        'LastName'  => [decode("iso-8859-1", "J\x{F6}rgensen"), 'Madsen'],
        'FirstName' => ["Jrg", 'Mads']
    }),
    'FirstName=J%C3%B6rg&FirstName=Mads&LastName=J%C3%B6rgensen&LastName=Madsen',
    'BuildParameterString array, umlaut, utf8');

is( BuildParameterString({
        'LastName'  => [decode("iso-8859-1", "J\x{F6}rgensen"), 'Madsen'],
        'FirstName' => ["Jrg", 'Mads']
    }, 1),
    'FirstName=J%F6rg&FirstName=Mads&LastName=J%F6rgensen&LastName=Madsen',
    'BuildParameterString array, umlaut, noutf8');

# StringIsURL
is(StringIsURL('my/dir/file.jpg'), 0, 'StringIsURL relative filepath');
is(StringIsURL('c:\\dir\\file.jpg'), 0, 'StringIsURL abs filepath');
is(StringIsURL('http_www_url.jpg'), 0, 'StringIsURL filename');
is(StringIsURL('www.myurl.com'), 0, 'StringIsURL www domain');
is(StringIsURL('myurl.com/my/dir'), 0, 'StringIsURL domain with subdir');
is(StringIsURL('http://myurl.com/my/dir'), 1, 'StringIsURL http domain with subdir');
is(StringIsURL('http://127.0.0.1'), 1, 'StringIsURL http localhost');
is(StringIsURL('https://www.mydomain.de'), 1, 'StringIsURL https domain');
is(StringIsURL('http://zufdev02'), 1, 'StringIsURL hostname');
