use Test::More tests => 9;

use strict;
use warnings;

use DE_EPAGES::Core::API::File qw( RunOnDirectory );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::Error qw( Error ExistsError );
use DE_EPAGES::Core::API::Warning qw( BlockWarnings );
use DE_EPAGES::Installer::API::CVSPatchHandler;

RunScript( Message => 'long', Sub => \&MainTest );

sub MainTest() {
    Test_FilterRdiffOutput();

    # make sure running Test is running in the correct directorys

    RunOnDirectory(
        $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/Installer/t/',
        sub { Test_getFileListUnchangedContent() }
    );
}


sub _cleanUpCVS ($$$$){

    my ($CVSPH, $PatchTagNew, $ReleaseTagNew, $File) = @_;

    my $loop = 3;
    my $done = 0;
    print 'Clean Up...';
    while ( $loop > 0 ) {
        eval {

            # delete tags
            $CVSPH->command( 'update', [ '-r', '1.3.2.2' ], [$File], {}, { 'outcontent' => 1 } );
            $CVSPH->command( 'tag', ['-d'], [ $PatchTagNew, $File ], {}, { 'outcontent' => 1 } );
            $CVSPH->command( 'update', [ '-r', '1.3.2.1' ], [$File], {}, { 'outcontent' => 1 } );
            $CVSPH->command( 'tag', [ '-d', $ReleaseTagNew ], [$File], {}, { 'outcontent' => 1 } );
        };

        if (not ExistsError()){
            $loop = 0;
            $done = 1;
        };
        $loop--;
        sleep(1);
    }
    print $done ? "done\n" : "fail. Run this script again.\n";


}

sub _moveCVSTags ($$$$){

    my ($CVSPH, $PatchTagNew, $ReleaseTagNew, $File) = @_;

    my $loop = 3;
    my $done = 0;

    print 'try to move tags...';
    while ( $loop > 0 ) {
        eval {
            $CVSPH->command( 'update', [ '-r', '1.3.2.1' ], [$File], {}, { 'outcontent' => 1 } );
            $CVSPH->command( 'tag', ['-d'], [ $PatchTagNew, $File ], {}, { 'outcontent' => 1 } );
            $CVSPH->command( 'update', [ '-r', '1.3.2.2' ], [$File], {}, { 'outcontent' => 1 } );
            $CVSPH->command( 'tag', [], [ $PatchTagNew, $File ], {}, { 'outcontent' => 1 } );
        };

        if ( not ExistsError() ) {
            $loop = 0;
            $done = 1;
        }
        $loop--;
        sleep(1);
    };
    print $done ? "done\n" : "fail\n";
    Error ('Moving tags in CVS failed. Corrupt state can\'t go on. Run this script again.') unless $done;


}

sub _prepareCVS($$$$){
    my ($CVSPH, $PatchTagNew, $ReleaseTagNew, $File) = @_;

    my $loop = 3;
    my $done = 0;

    print 'try init CVS...';
    while ( $loop > 0 ) {
        eval {

            # Clean Up CVS - to get a guranteed defined starting point
            $CVSPH->command( 'update', [ '-r', '1.3.2.2' ], [$File], {}, { 'outcontent' => 1 } );
            $CVSPH->command( 'tag', ['-d'], [ $PatchTagNew, $File ], {}, { 'outcontent' => 1 } );
            $CVSPH->command( 'tag', [ '-d', $ReleaseTagNew ], [$File], {}, { 'outcontent' => 1 } );
            $CVSPH->command( 'update', [ '-r', '1.3.2.1' ], [$File], {}, { 'outcontent' => 1 } );
            $CVSPH->command( 'tag', [ '-d', $ReleaseTagNew ], [$File], {}, { 'outcontent' => 1 } );
            $CVSPH->command( 'tag', ['-d'], [ $PatchTagNew, $File ], {}, { 'outcontent' => 1 } );

            # setup ReleaseTagNew & PatchTagNew
            $CVSPH->command( 'update', [ '-r', '1.3.2.1' ], [$File], {}, { 'outcontent' => 1 } );
            $CVSPH->command( 'tag', [$ReleaseTagNew], [$File], {}, { 'outcontent' => 1 } );
            $CVSPH->command( 'tag', [$PatchTagNew],   [$File], {}, { 'outcontent' => 1 } );
        };

        if (not ExistsError()){
            $loop = 0;
            $done = 1;
        };
        $loop--;
        sleep(1);
    }
    print $done ? "done\n" : "fail\n";

    Error ('CVS initialization failed. Corrupt state can\'t go on. Run this script again.') if (not $done);
}
#========================================================================================
# function     Test_FilterRdiffOutput
# state        public
#----------------------------------------------------------------------------------------
# syntax       Test_FilterRdiffOutput(  );
#----------------------------------------------------------------------------------------
# description  Testing the filter mechanism with static output from rdiff command. No
#               CVS connection is needed.
#========================================================================================
sub Test_FilterRdiffOutput() {

    # Tests with static data from real "cvs rdiff -s -r version_A -r version_B"

    #
    #   Preperation
    #

    my $rdiffOutput =
        'File RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Presentation/Scripts/Sprites/'
      . 'sprite.pl changed from revision 1.7 to 1.9' . "\n"
      . 'File RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Presentation/Scripts/util/yuicompressor/'
      . 'yuicompressor-2.4.2.jar is removed; not included in release tag release_6_12_0' . "\n"
      . 'File RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Presentation/Scripts/util/yuicompressor/'
      . 'yuicompressor-2.4.6.jar is new; current revision 1.1' . "\n";

    # $cvsdirectory will be truncated from absolute path to generate a relative path
    # for an abitary position in the repository
    my $cvsdirectory = 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Presentation/Scripts';

    my $CVSPatchHandler = DE_EPAGES::Installer::API::CVSPatchHandler->new( undef, { print => 0 } );

    #
    #   Testing error handling
    #

    eval { $CVSPatchHandler->_FilterRdiffOutput( $rdiffOutput, $cvsdirectory, {} ) };
    ok( ExistsError(), 'throw error if filter is not set correctly (1)' );
    eval {
        $CVSPatchHandler->_FilterRdiffOutput( $rdiffOutput, $cvsdirectory,
            { 'new' => 0, 'changed' => 0, 'removed' => 0 } );
    };
    ok( ExistsError(), 'throw error if filter is not set correctly (2)' );

    #
    #   Check Filter Output
    #

    my @lines = ();
    @lines = $CVSPatchHandler->_FilterRdiffOutput( $rdiffOutput, $cvsdirectory, { 'new' => 1 } );
    ok(
        (
                  scalar @lines == 1
              and $lines[0] =~ m/\Qyuicompressor-2.4.6.jar\E$/
              and $lines[0] !~ m/\Q$cvsdirectory\E/
        ),
        'select only new files'
    );

    @lines = ();
    @lines =
      $CVSPatchHandler->_FilterRdiffOutput( $rdiffOutput, $cvsdirectory,
        { 'new' => 1, 'changed' => 1 } );
    ok(
        (
                  scalar @lines == 2
              and $lines[1] =~ m/\Qyuicompressor-2.4.6.jar\E$/
              and $lines[0] =~ m/\Qsprite.pl\E/
        ),
        'select only new & changed files'
    );

    @lines = ();
    @lines =
      $CVSPatchHandler->_FilterRdiffOutput( $rdiffOutput, $cvsdirectory,
        { 'new' => 1, 'changed' => 1, 'removed' => 1 } );
    ok(
        (
                  scalar @lines == 3
              and $lines[2] =~ m/\Qyuicompressor-2.4.6.jar\E$/
              and $lines[0] =~ m/\Qsprite.pl\E/
              and $lines[1] =~ m/\Qyuicompressor-2.4.2.jar\E$/

        ),
        'select new, changed and removed files'
    );

}


#========================================================================================
# function     Test_getFileListUnchangedContent
# state        public_private
#----------------------------------------------------------------------------------------
# syntax       Test_getFileListUnchangedContent( );
#----------------------------------------------------------------------------------------
# description  Tests running with live CVS repository; CVS must be accesseble and the $File must exist in right revisions.
#               The Commands 'cvs update' and 'cvs tag' may fail depending on load situation of the cvs servers. This is by passed
#               using eval loops which issue an command up to 3 times before an error is thrown.
#========================================================================================
sub Test_getFileListUnchangedContent () {

    #
    #   Preperation
    #

    # Create object; set print true for debug output of cvs commands
    my $CVSPH = DE_EPAGES::Installer::API::CVSPatchHandler->new( undef, { print => 0 } );
    eval {
        # Is CVS Responding
        my $CVSResponse = $CVSPH->command( 'ls', ['RND/'], [], {}, { 'outcontent' => 1 } );
        ok( $CVSResponse->{outcontent}, 'CVS is responding' );
    };

    Error('CVS is not responding. Can\'t proceed.') if ExistsError();

    # Tag names
    my $File          = 'test_CVSHandler.txt';
    my $ReleaseTagOld = 'release_test_CVSHandler_old';    # tag already exists in CVS
    my $ReleaseTagNew = 'release_test_CVSHandler_new';    # will be added and later deleted
    my $PatchTagNew   = 'patch_test_CvSHandler_new';      # will be added and later deleted


    _prepareCVS($CVSPH, $PatchTagNew, $ReleaseTagNew, $File);


    #
    #   Wrong CVS directory as imput parameter -> Error
    #
    my $isErrorDetected;
    my $UnchangedFiles;
    eval {
        $UnchangedFiles = $CVSPH->getFileListUnchangedContent(
            '1.3', '1.3',
            {
                'cvsdirectory'  => 'RND/wrong/path',    # provokes an error, revision don't matter
                'patch_tag_new' => '1.3'
            }
        );
    };

    if ( ExistsError() ) {
        $isErrorDetected = 1;
    } else {
        $isErrorDetected = 0;
    }
    ok( $isErrorDetected, 'getFileListUnchangedContent throws error if CVScommand return error' );

    #
    #   Negativ Test Empty result returned -> Warning, possible an error
    #

    # prevents logging of this warning and catch warning content
    my $aWarnings = BlockWarnings(
        sub {

            # rev 1.1<->1.3.2.1 and 1.1<->1.3.2.1
            $UnchangedFiles = $CVSPH->getFileListUnchangedContent(
                $ReleaseTagOld,
                $ReleaseTagNew,
                {
                    'cvsdirectory' => 'RND/epages/epages5/AllOS/Cartridges/'
                      . 'DE_EPAGES/Installer/t/',
                    'patch_tag_new' => $PatchTagNew
                }
            );
        },
        ['Warning']
    );
    ok( ( scalar(@$aWarnings) == 1 and scalar(@$UnchangedFiles) == 0 ),
        'getFileListUnchangedContent throws warning for zero result and returns a empty list' );

    #
    #   Positiv Test - diffrent revisions and diffrent content -> zero unchanged files
    #

    _moveCVSTags($CVSPH, $PatchTagNew, $ReleaseTagNew, $File);

    # rev 1.1<->1.3.2.1 and 1.1<->1.3.2.2
    @$UnchangedFiles = $CVSPH->getFileListUnchangedContent(
        $ReleaseTagOld,
        $ReleaseTagNew,
        {
            'cvsdirectory'  => 'RND/epages/epages5/AllOS/Cartridges/DE_EPAGES/Installer/t/',
            'patch_tag_new' => $PatchTagNew
        }
    );

    ok( scalar(@$UnchangedFiles) == 1, 'getFileListUnchangedContent -> one unachanged file' );

    #
    #  Clean Up
    #

    _cleanUpCVS($CVSPH, $PatchTagNew, $ReleaseTagNew, $File);

}
