use strict;

use DE_EPAGES::Core::API::File qw ( GetFileContent );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::MeinPaket::API::DTO::Attribute;
use DE_EPAGES::MeinPaket::API::DTO::AttributeValue;
use DE_EPAGES::MeinPaket::API::DTO::CatalogStructure;
use DE_EPAGES::MeinPaket::API::DTO::Descriptions;
use DE_EPAGES::MeinPaket::API::DTO::DownloadRequest;
use DE_EPAGES::MeinPaket::API::DTO::Header;
use DE_EPAGES::MeinPaket::API::DTO::Image;
use DE_EPAGES::MeinPaket::API::DTO::ProductDescription;
use DE_EPAGES::MeinPaket::API::DTO::UploadRequest;
use DE_EPAGES::MeinPaket::API::DTO::Offers;
use DE_EPAGES::MeinPaket::API::DTO::ProductOffer;
use DE_EPAGES::MeinPaket::API::DTO::Categories;
use DE_EPAGES::MeinPaket::API::DTO::Category;
use DE_EPAGES::MeinPaket::API::DTO::QueryRequest;
use DE_EPAGES::MeinPaket::API::DTO::Orders;

use DE_EPAGES::XML::API::Tidy qw ( tidy );
use Test::More;
use Readonly;

# directory to MeinPaket.de sample files.
Readonly my $XMLSAMPLEDIR => $ENV{'EPAGES_CARTRIDGES'} . '/DE_EPAGES/MeinPaket/t/Samples/';

sub Main {
    testDownloadCategories();
    testDescriptions();
    testUploadOffers();
    testUploadCompleteProduct();
    testUploadCategories();
    testDownloadOrders();
    done_testing();
}

sub testDescriptions {    ## no critic (ProhibitFunctionSize)
    my $Request = DE_EPAGES::MeinPaket::API::DTO::UploadRequest->new();
    my $Header  = DE_EPAGES::MeinPaket::API::DTO::Header->new(
        login    => 'ihre.emailadresse@ihrdomain.de',
        password => 'Pa55word',
        language => 'de',
    );
    $Request->header($Header);
    my $Body = DE_EPAGES::MeinPaket::API::DTO::Descriptions->new();
    my $Desc = DE_EPAGES::MeinPaket::API::DTO::ProductDescription->new(
        productId        => '1234',
        ean              => '1234567890123',
        manufacturerName => 'Herstellername',
        manufacturerPN   => '123',
        name             => 'Produktname für die Anzeige',
        shortDescription => 'Dieser Text für die Listenansichten verwendet.',
        longDescription  => 'Dieser Text wird bei der Produktdetailansicht ',
    );
    $Desc->pushImage(
        DE_EPAGES::MeinPaket::API::DTO::Image->new(
            url     => 'http://www.ihrshop.com/pfad/zu/Ihrem/Produkbild/1234.jpg',
            caption => 'Bildbeschreibung'
        )
    );
    $Desc->marketplaceCategory('1.1.3.4');
    my $Attribute = DE_EPAGES::MeinPaket::API::DTO::Attribute->new(
        code => 'Att001',
        name => 'Erdbeeren'
    );
    $Attribute->pushValue(
        DE_EPAGES::MeinPaket::API::DTO::AttributeValue->new(
            unit  => 'mg',
            value => '30',
        )
    );
    $Desc->pushAttribute($Attribute);
    my $Attribute2 = DE_EPAGES::MeinPaket::API::DTO::Attribute->new(
        code => 'Att002',
        name => 'Wasserstoff'
    );
    $Attribute2->pushValue(
        DE_EPAGES::MeinPaket::API::DTO::AttributeValue->new(
            unit  => 'g',
            value => '45',
        )
    );
    $Desc->pushAttribute($Attribute2);
    my $Attribute3 = DE_EPAGES::MeinPaket::API::DTO::Attribute->new(
        code => 'Att003',
        name => 'Aceton'
    );
    $Attribute3->pushValue( DE_EPAGES::MeinPaket::API::DTO::AttributeValue->new( value => '-', ) );
    $Desc->pushAttribute($Attribute3);
    $Body->pushProductDescription($Desc);
    $Request->descriptions($Body);

    is(
        tidy( $Request->toXML() ),
        tidy( ${ GetFileContent( $XMLSAMPLEDIR . 'mp_example_20_02_description.xml', { 'encoding' => 'utf-8' } ) } ),
        'toXML: uploadDescriptions'
    );
}

sub testDownloadCategories {
    my $Request = DE_EPAGES::MeinPaket::API::DTO::DownloadRequest->new();
    my $Header  = DE_EPAGES::MeinPaket::API::DTO::Header->new(
        login    => 'ihre.emailadresse@ihrdomain.de',
        password => 'Pa55word',
        language => 'de',
    );
    $Request->header($Header);
    $Request->catalogstructure( DE_EPAGES::MeinPaket::API::DTO::CatalogStructure->new() );
    is( tidy( $Request->toXML() ), tidy( ${ GetFileContent( $XMLSAMPLEDIR . 'mp_example_20_01_download_categories.xml' ) } ), 'toXML: downloadCategories' );
}

sub testUploadOffers {
    my $Request = DE_EPAGES::MeinPaket::API::DTO::UploadRequest->new();
    my $Header  = DE_EPAGES::MeinPaket::API::DTO::Header->new(
        login    => 'ihre.emailadresse@ihrdomain.de',
        password => 'Pa55word',
        language => 'de',
    );
    $Request->header($Header);
    my $Body = DE_EPAGES::MeinPaket::API::DTO::Offers->new();
    $Body->pushProductOffer(
        DE_EPAGES::MeinPaket::API::DTO::ProductOffer->new(
            productId    => '1234',
            price        => '39.90',
            taxGroup     => 'Reduced',
            availability => 20,
            deliverytime => 4,
        )
    );
    $Request->offers($Body);
    is( tidy( $Request->toXML() ), tidy( ${ GetFileContent( $XMLSAMPLEDIR . 'mp_example_20_03_offer.xml' ) } ), 'toXML: uploadOffers' );
}

sub testUploadCompleteProduct {
    my $Request = DE_EPAGES::MeinPaket::API::DTO::UploadRequest->new();
    my $Header  = DE_EPAGES::MeinPaket::API::DTO::Header->new(
        login    => 'ihre.emailadresse@ihrdomain.de',
        password => 'Pa55word',
        language => 'de',
    );
    $Request->header($Header);
    my $Body = DE_EPAGES::MeinPaket::API::DTO::Descriptions->new();

    my $Desc = DE_EPAGES::MeinPaket::API::DTO::ProductDescription->new(
        productId        => '1234',
        ean              => '1234567890123',
        manufacturerName => 'Herstellername',
        manufacturerPN   => '123',
        name             => 'Produktname für die Anzeige',
        shortDescription => 'Dieser Text für die Listenansichten verwendet.',
        longDescription  => 'Dieser Text wird bei der Produktdetailansicht ',
    );

    $Desc->pushImage(
        DE_EPAGES::MeinPaket::API::DTO::Image->new(
            url     => 'http://www.ihrshop.com/pfad/zu/Ihrem/Produkbild/1234.jpg',
            caption => 'Bildbeschreibung'
        )
    );
    $Desc->marketplaceCategory('1.1.3.4');
    my $Attribute = DE_EPAGES::MeinPaket::API::DTO::Attribute->new(
        code => 'Att001',
        name => 'Erdbeeren'
    );
    $Attribute->pushValue(
        DE_EPAGES::MeinPaket::API::DTO::AttributeValue->new(
            unit  => 'mg',
            value => '30',
        )
    );
    $Desc->pushAttribute($Attribute);
    my $Attribute2 = DE_EPAGES::MeinPaket::API::DTO::Attribute->new(
        code => 'Att002',
        name => 'Wasserstoff'
    );
    $Attribute2->pushValue(
        DE_EPAGES::MeinPaket::API::DTO::AttributeValue->new(
            unit  => 'g',
            value => '45',
        )
    );
    $Desc->pushAttribute($Attribute2);
    my $Attribute3 = DE_EPAGES::MeinPaket::API::DTO::Attribute->new(
        code => 'Att003',
        name => 'Aceton'
    );
    $Attribute3->pushValue( DE_EPAGES::MeinPaket::API::DTO::AttributeValue->new( value => '-', ) );
    $Desc->pushAttribute($Attribute3);
    $Body->pushProductDescription($Desc);

    $Request->descriptions($Body);

    $Body = DE_EPAGES::MeinPaket::API::DTO::Offers->new();
    $Body->pushProductOffer(
        DE_EPAGES::MeinPaket::API::DTO::ProductOffer->new(
            productId    => '1234',
            price        => '39.90',
            taxGroup     => 'Reduced',
            availability => 20,
            deliverytime => 4,
        )
    );
    $Request->offers($Body);

    is(
        tidy( $Request->toXML() ),
        tidy( ${ GetFileContent( $XMLSAMPLEDIR . 'mp_example_20_04_complete_product.xml', { 'encoding' => 'utf-8' } ) } ),
        'toXML: uploadCompleteProduct'
    );
}

sub testUploadCategories {
    my $Request = DE_EPAGES::MeinPaket::API::DTO::UploadRequest->new();
    my $Header  = DE_EPAGES::MeinPaket::API::DTO::Header->new(
        login    => 'ihre.emailadresse@ihrdomain.de',
        password => 'Pa55word',
        language => 'de',
    );
    $Request->header($Header);

    my $Categories = DE_EPAGES::MeinPaket::API::DTO::Categories->new();
    $Categories->pushCategory(
        DE_EPAGES::MeinPaket::API::DTO::Category->new(
            code             => 'F',
            name             => 'Reiseführer',
            shortDescription => 'Reiseführer für Länder und Städte',
        )
    );
    $Categories->pushCategory(
        DE_EPAGES::MeinPaket::API::DTO::Category->new(
            code             => 'FP',
            name             => 'Portugal',
            shortDescription => 'Reiseführer von Portugal',
            parentCode       => 'F',
        )
    );
    $Categories->pushCategory(
        DE_EPAGES::MeinPaket::API::DTO::Category->new(
            code             => 'FPL',
            name             => 'Lissabon',
            shortDescription => 'Reiseführer von Lissabon',
            parentCode       => 'FP',
        )
    );
    $Categories->pushCategory(
        DE_EPAGES::MeinPaket::API::DTO::Category->new(
            code             => 'FPS',
            name             => 'Sintra',
            shortDescription => 'Reiseführer von Sintra',
            parentCode       => 'FP',
        )
    );

    $Request->categories($Categories);

    is(
        tidy( $Request->toXML() ),
        tidy( ${ GetFileContent( $XMLSAMPLEDIR . 'mp_example_20_07_categories.xml', { 'encoding' => 'utf-8' } ) } ),
        'toXML: uploadCategories'
    );
}

sub testDownloadOrders {
    my $Request = DE_EPAGES::MeinPaket::API::DTO::QueryRequest->new();
    my $Header  = DE_EPAGES::MeinPaket::API::DTO::Header->new(
        login    => 'ihre.emailadresse@ihrdomain.de',
        password => 'Pa55word',
        language => 'de',
    );

    my $QueryOrders = DE_EPAGES::MeinPaket::API::DTO::Orders->new(
        dateFrom => '2010-10-20T06:00:00',
        dateTo   => '2010-10-27T14:59:59',
    );

    $Request->header($Header);
    $Request->orders($QueryOrders);

    is(
        tidy( $Request->toXML() ),
        tidy( ${ GetFileContent( $XMLSAMPLEDIR . 'mp_example_30_01_download_orders.xml', { 'encoding' => 'utf-8' } ) } ),
        'toXML: downloadOrders'
    );
}

RunScript( Sub => \&Main );
