use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
  ExistsObjectByPath
  LoadObjectByPath
  LoadRootObject
  LoadClassByAlias
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use Readonly;
use Test::More;

Readonly my $ROOTPATH => '/Test';

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # startUp
            ok( Init(), 'initialization' );

            # tests
            eval { 
                TestAffiliateVisibilityCategories();
                TestAffiliateVisibilityProducts();
            };

            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

sub TestAffiliateVisibilityCategories {
    my $Shop = LoadObjectByPath('/Test/TestMasterShop');
    my $RootCat = $Shop->get('Categories');
    is($RootCat->get('IsAffiliateVisible'),undef,'Categories: IsAffiliateVisible=undef');
    my $Tents = $RootCat->child('Tents');
    is($Tents->get('IsAffiliateVisible'),1,'Tents: IsAffiliateVisible=1');
    my $FamilyTents = $Tents->child('FamilyTents');
    is($FamilyTents->get('IsAffiliateVisible'),1,'FamilyTents: IsAffiliateVisible=1');
    $Tents->set({'IsAffiliateVisible' => 0 });
    is($Tents->get('IsAffiliateVisible'),0,'Tents: IsAffiliateVisible=0');
    is($FamilyTents->get('IsAffiliateVisible'),0,'FamilyTents: IsAffiliateVisible=0 (recursive)');
    my $TestTents = $FamilyTents->insertChild('Category',{
        'Alias' => 'TestTents',
    });
    is($TestTents->get('IsAffiliateVisible'),0,'TestTents: IsAffiliateVisible=0 (recursive two levels)');
    $FamilyTents->set({'IsAffiliateVisible' => 1 });
    is($TestTents->get('IsAffiliateVisible'), 1,'TestTents: IsAffiliateVisible=1 (recursive one level)');
    $FamilyTents->set({'IsAffiliateVisible' => undef });
    is($TestTents->get('IsAffiliateVisible'), 0,'TestTents: IsAffiliateVisible=1 (recursive two levels again)');

    # check the map directly
    my $AffiliateMapClass = LoadClassByAlias('AffiliateVisibilityMap');
    is(scalar @{$AffiliateMapClass->getObjects('ByMasterShopContentObjectID', [ $TestTents->id ])}, 0, 'TestTents: No MapObject');
    is(scalar @{$AffiliateMapClass->getObjects('ByMasterShopContentObjectID', [ $FamilyTents->id ])}, 0, 'FamilyTents: No MapObject');
    is(scalar @{$AffiliateMapClass->getObjects('ByMasterShopContentObjectID', [ $Tents->id ])}, 1, 'Tents: One MapObject');
}

sub TestAffiliateVisibilityProducts {
    my $Shop = LoadObjectByPath('/Test/TestMasterShop');
    my $Folder = $Shop->child('Products');
    my $be_40401 = $Folder->child('be_40401');
    is($be_40401->getWithParent('IsAffiliateVisible'), 1,'be_40401: IsAffiliateVisible=1 (by default)');
    my $be_4040104003 = $be_40401->child('SubProducts')->child('be_4040104003');
    is($be_4040104003->getWithParent('IsAffiliateVisible'), 1,'be_4040104003: IsAffiliateVisible=1 (getWithParent)');
    $be_40401->set({'IsAffiliateVisible' => 0 });
    is($be_40401->getWithParent('IsAffiliateVisible'), 0,'be_40401: IsAffiliateVisible=0 (set)');
    is($be_4040104003->getWithParent('IsAffiliateVisible'), 0,'be_4040104003: IsAffiliateVisible=0 (getWithParent)');


    # check the map directly
    my $AffiliateMapClass = LoadClassByAlias('AffiliateVisibilityMap');
    is(scalar @{$AffiliateMapClass->getObjects('ByMasterShopContentObjectID', [ $be_40401->id ])}, 1, 'be_40401: One MapObject');
    is(scalar @{$AffiliateMapClass->getObjects('ByMasterShopContentObjectID', [ $be_4040104003->id ])}, 0, 'be_4040104003: No MapObject');

}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'AffiliateVisibilityMap.xml', $ROOTPATH );
    return 1;
}

RunScript( Sub => \&Main );

1;