use strict;
use Test::More;
use Time::HiRes qw (time);
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Core::API::File qw(WriteFile);
use DE_EPAGES::Core::API::Unicode qw( Unicode2Filename );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( ExistsObjectByPath LoadObjectByPath);
use DE_EPAGES::WebInterface::API::FakeRequest;

#========================================================================================
# function     TimeThis
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Time = TimeThis( $Code );
#----------------------------------------------------------------------------------------
# description  Executes a code block and returns the execution time.
#----------------------------------------------------------------------------------------
# input        $Code | code to evaluate | code reference
# return       $Time | execution time in seconds | float
#========================================================================================
sub TimeThis {
    my ($Code) = @_;

    my $time0 = time();
    $Code->();
    return time() - $time0;
}

#========================================================================================
# function     Main
# state        private
#========================================================================================
sub Main {

    ## no critic qw(ProhibitCaptureWithoutTest RequireExtendedFormatting ProhibitMagicNumbers ProhibitLongLines)

    my $NumTests = 27;
    plan tests => $NumTests;

    my $ObjectPath = '/Shops/DemoShop';
    my $DemoShopURL;
    my $ProductID;
    my $ProductURL;
    my $IsPageCacheUsed;
    my $LocaleID;
    RunOnStore(
        Store      => GetTestSite(),
        Sub        => sub {
            if( ExistsObjectByPath( $ObjectPath ) ) {
                my $Shop = LoadObjectByPath( $ObjectPath );
                $DemoShopURL = $Shop->get('WebUrl');
                $IsPageCacheUsed = $Shop->get('IsPageCacheUsed');
                $LocaleID = $Shop->get('LocaleID');

                my $Product = $Shop->get('ProductFolder')->child('ho_40407')->child('SubProducts')->child('ho_4040705005');
                $ProductID = $Product->id;
                $ProductURL = $Product->get('WebUrl');
            }
        }
    );

    SKIP: {
        skip 'no DemoShop installed', $NumTests unless defined $DemoShopURL;

        my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
        my $GetURL = sub {
            my ($URL) = @_;

            my $Response = $FakeRequest->get( $URL );
            if( !$Response->is_success ) {
                die "Can't load '$URL': " . $Response->status_line;
            }
            else {
                return $Response->content;
            }
        };
        my $GetTitle = sub {
            my ($URL) = @_;

            my $Content = $GetURL->( $URL );
            if( $Content =~ /<title.*?>(.*?)<\/title>/ ) {
                return $1;
            }
            else {
                die "No title found in '$URL'";
            }
        };

        ok( !$IsPageCacheUsed, 'Page Cache is disabled' );
        is( $LocaleID, 'en_GB', 'default Locale is en_GB' );

        my $AddToBasketURL = $ProductURL . '&ChangeAction=AddToBasket&Quantity=2&ViewObjectID='.$ProductID;

        my $PreloadTime = TimeThis( sub {
            is( $GetTitle->( $DemoShopURL ), 'Milestones - Equipped to meet your goals', 'preload start page' );
            is( $GetTitle->( $ProductURL ), 'Jack Wolfskin Blizzard Jacket - Milestones', 'preload product page' );

            my $AddToBasketContent = $GetURL->( $AddToBasketURL );
            _like( $AddToBasketContent, qr!Total amount!, 'preload add to basket' );

            my $BasketID = _BasketID( $AddToBasketContent );
            my $BaseURL = _BaseURL( $AddToBasketContent );

            my $BasketURL = "$BaseURL?ObjectID=$BasketID";
            my $BasketContent = $GetURL->($BasketURL );
            _like( $BasketContent, qr/Order and delivery comments/, 'preload basket page' );

            my $LoginURL = $BaseURL.'?ObjectID='.$BasketID.'&ChangeAction=SaveLoginForm&Login=mmustermann&Password=123456';
            my $LoginContent = $GetURL->( $LoginURL );
            _like( $LoginContent, qr/Max Mustermann/, 'preload login' );

            $BaseURL = _BaseURL($LoginContent);

            my $AddressURL = $BaseURL.'?ObjectID='.$BasketID.'&ViewAction=ViewAddress';
            my $AddressContent = $GetURL->( $AddressURL );
            _like($AddressContent , qr/Add a new delivery address/, 'preload address selection' );

            $AddressContent =~ /name="BillingAddress" value="(\d+)"/ or die 'BillingAddress id not found';
            my $BillingAddressID = $1;

            my $ConfirmURL = "$BaseURL?ObjectID=$BasketID&ChangeAction=SelectAddress&ChangeAction=DoOrder&BillingAddress=$BillingAddressID&ShippingAddress=$BillingAddressID";
            _like( $GetURL->( $ConfirmURL ), qr/Place order/, 'preload confirmation page' );

            my $OrderURL = $BaseURL.'?ObjectID='.$BasketID.'&ChangeAction=TestTermsAndConditions&tac=1&ChangeAction=DoOrder';
            _like( $GetURL->( $OrderURL ), qr/Confirmation/, 'preload order' );
        } );
        cmp_ok( $PreloadTime, '<', 20, sprintf('preload: %0.2f s', $PreloadTime) );

#DB::enable_profile();
        my $StartPageTime = TimeThis( sub {
            is( $GetTitle->( $DemoShopURL ), 'Milestones - Equipped to meet your goals', 'start page' );
        } );
        cmp_ok( $StartPageTime, '<', 0.8, sprintf('start page: %0.2f s', $StartPageTime) );
#DB::disable_profile();

        my $ProductPageTime = TimeThis( sub {
            is( $GetTitle->( $ProductURL ), 'Jack Wolfskin Blizzard Jacket - Milestones', 'preload product page' );
        } );
        cmp_ok( $ProductPageTime, '<', 1, sprintf('product page: %0.2f s', $ProductPageTime) );

        # add to basket
        my $BaseURL;
        my $BasketID;
        my $AddToBasketTime = TimeThis( sub {
            my $AddToBasketContent = $GetURL->( $AddToBasketURL );
            _like( $AddToBasketContent, qr!Total amount!, 'add to basket' );

            $BasketID = _BasketID( $AddToBasketContent );
            $BaseURL = _BaseURL( $AddToBasketContent );
        } );
        cmp_ok( $AddToBasketTime, '<', 2, sprintf('add to basket: %0.2f s', $AddToBasketTime) );

        my $BasketURL = $BaseURL.'?ObjectID='.$BasketID;
        my $BasketTime = TimeThis( sub {
            my $BasketContent = $GetURL->($BasketURL );
            _like( $BasketContent, qr/Order and delivery comments/, 'basket page' );
        });
        cmp_ok( $BasketTime, '<', 1, sprintf('basket: %0.2f s', $BasketTime) );

        my $LoginTime = TimeThis( sub {
            my $LoginURL = $BaseURL.'?ObjectID='.$BasketID.'&ChangeAction=SaveLoginForm&Login=mmustermann&Password=123456';
            my $LoginContent = $GetURL->( $LoginURL );
            _like( $LoginContent, qr/Max Mustermann/, 'login' );

            $BaseURL = _BaseURL($LoginContent);
        });
        cmp_ok( $LoginTime, '<', 1.5, sprintf('login: %0.2f s', $LoginTime) );

        my $BillingAddressID;
        my $AddressTime = TimeThis( sub {
            my $AddressURL = $BaseURL.'?ObjectID='.$BasketID.'&ViewAction=ViewAddress';
            my $AddressContent = $GetURL->( $AddressURL );
            _like($AddressContent , qr/Add a new delivery address/, 'address selection' );

            $AddressContent =~ /name="BillingAddress" value="(\d+)"/ or die 'BillingAddress id not found';
            $BillingAddressID = $1;
        });
        cmp_ok( $AddressTime, '<', 0.8, sprintf('address: %0.2f s', $AddressTime) );

        my $ConfirmTime = TimeThis( sub {
            my $ConfirmURL = "$BaseURL?ObjectID=$BasketID&ChangeAction=SelectAddress&ChangeAction=DoOrder&BillingAddress=$BillingAddressID&ShippingAddress=$BillingAddressID";
            _like( $GetURL->( $ConfirmURL ), qr/Place order/, 'confirmation page' );
        });
        cmp_ok( $ConfirmTime, '<', 1, sprintf('confirm: %0.2f s', $ConfirmTime) );

        my $OrderTime = TimeThis( sub {
            my $OrderURL = $BaseURL.'?ObjectID='.$BasketID.'&ChangeAction=TestTermsAndConditions&tac=1&ChangeAction=DoOrder';
            _like( $GetURL->( $OrderURL ), qr/Confirmation/, 'order' );
        });
        cmp_ok( $OrderTime, '<', 1.5, sprintf('order: %0.2f s', $OrderTime) );
    };

    return;
}

sub _like($$$) {
    my ($Content, $RegExp, $Message) = @_;

    my $ok = $Content =~ $RegExp;
    ok( $ok, $Message );

    unless ($ok) {
        my $FileName = Unicode2Filename($Message).'.html';
        WriteFile( $FileName, \$Content );
        diag "Content does not match $RegExp";
        note "Content was saved as $FileName";
    }
}

sub _BasketID {
    my ($Content) = @_;

    if( $Content =~ m!<a\ class="MiniCheckoutLink\ .*?Basket.*?"\ href="\?ObjectID=(\d+)">!x ) {
        return $1;
    }
    die 'Basket link not found';
}

sub _BaseURL {
    my ($Content) = @_;

    if( $Content =~ m!<base\s+href="(.*?)"!x ) {
        return $1;
    }
print $Content;
    die 'base tag not found';
}


RunScript( Sub => \&Main, Debug => 1 );
