
use strict;
use Test::More tests => 20;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Customer::API::Shop qw ( GetNextCustomerNo GetCustomerFolder );
use DE_EPAGES::DataCache::API::Cache qw( UpdateLocalConnection );
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject ExistsObjectByPath LoadObjectByPath);
use DE_EPAGES::Order::API::Shop qw ( GetNextOrderNo );

local $ENV{'EPAGES_TESTSITE'} = "Store" unless $ENV{'EPAGES_TESTSITE'};
my $ROOTPATH = '/Test';

sub Main {

    ConnectTestSite();

    # Setup database; usually each db section should contain an "key" entry where the aes can be found, like:
    #
    #   [Connection]
    #   datasource=dbi:mysql
    #   login=usr_sitedb
    #   passwd=epages
    #   active=1
    #   Database=sitedb
    #   StoreType=Site
    #   key=hst58a42hvbt8671loegysq87n4mc7z1
    #
    # if not, we add one key here.
    my $dbi = GetCurrentDBHandle();
    $dbi->{'key'} = "hst58a42hvbt8671loegysq87n4mc7z1" unless $dbi->{'key'};

    # create test classes
    ok( Init(), "initialization" );

    my $Shop   = LoadObjectByPath( $ROOTPATH . '/TestShop1' );
    ok( $Shop->id, "create shop " . $Shop->id );

    my $CustomerNo = GetNextCustomerNo($Shop);
    ok( length $CustomerNo >= 4, "customer no. starts with 1000" );

    my $CustomerFolder = GetCustomerFolder($Shop);
    ok( !$CustomerFolder->existsChild($CustomerNo), "customer no. does not exists yet" );

    my $Customer = $Shop->get('CustomerClass')->insertObject( {
            Parent => $CustomerFolder,
            Alias  => $CustomerNo,
        }
    );
    ok( $Customer->id, "insert new customer" );
    ok( $Customer->alias eq $CustomerNo, "Customer.Alias" );

    my $OrderNo = GetNextOrderNo($Shop);
    ok( length $OrderNo >= 4, "order no. starts with 1000" );

    my $Order = $Customer->folder('Orders')->insertChild(
        'CustomerOrder', {
            'Alias'    => $OrderNo,
            'Shop'     => $Shop,
            'Customer' => $Customer,
        }
    );

    # add a line item container
    my $Container = $Order->insertChild(
        'LineItemContainer', {
            'Alias'      => 'LineItemContainer',
            'LanguageID' => $Shop->get('LanguageID'),
            'LocaleID'   => $Shop->get('LocaleID'),
            'CurrencyID' => $Shop->get('CurrencyID'),
            'TaxArea'    => $Shop->get('TaxMatrix')->defaultTaxArea,
            'TaxModel'   => $Shop->get('TaxModel'),
        }
    );
    $Order->set( { 'LineItemContainer' => $Container } );

    foreach my $TryNumber ( 1 .. 2 ) {
        my $LineItemPaymentDirectDebit = $Order->insertChild(
            'LineItemPaymentDirectDebit', {
                'Alias'             => '08/15 ' . $TryNumber,
                'BankAccountHolder' => 'Max Mustermann',
                'BankAccountNo' => '1234567890',
                'BankCode' => '10020055',
                'BankName' => 'Musterbank',
                'LineItemContainer' => $Container,
            }
        );

        # EPG-15717: don't get value from cache (for every second try)
        UpdateLocalConnection( $dbi->cacheKey ) if 0 == $TryNumber % 2;

        # Get methods should decrypt the data transparently.
        is( $LineItemPaymentDirectDebit->get('BankAccountNo'), '1234567890',
            "BankAccountNo $TryNumber" );
        is( $LineItemPaymentDirectDebit->get('BankName'), 'Musterbank', "BankName $TryNumber" );

        # Get the encrypted data from DB directly.
        my $SQL = "SELECT bankaccountno, bankname FROM lineitempaymentdirectdebit WHERE lineitempaymentddbtid = ?";
        my $aaRows = GetCurrentDBHandle()->execute( $SQL, [ $LineItemPaymentDirectDebit->id ] );

        is( length $aaRows->[0]->[0], 64, "BankAccountNo encrypted $TryNumber" );
        is( length $aaRows->[0]->[1], 64, "BankName encrypted $TryNumber" );

        # Update test
        # Get methods should decrypt the data transparently.
        $LineItemPaymentDirectDebit->set({'BankAccountNo' => '0987654321'});
        is( $LineItemPaymentDirectDebit->get('BankAccountNo'), '0987654321',
            "BankAccountNo Update");

        $LineItemPaymentDirectDebit->set({'BankName' => 'Muster_Bank'});
        is( $LineItemPaymentDirectDebit->get('BankName'), 'Muster_Bank', "BankName Update" );
    }

    ok( Cleanup(), "cleanup" );

    DisconnectSite();
}

sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

sub Init {
    return 1 if LoadRootObject()->existsChild('Test') && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    my $Folder = LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    $Folder->insertChild( 'Shop', { Alias => 'TestShop1' } );

    $Folder->insertChild(
        'Address', {
            'Alias'   => 'O1',
            'Street'  => 'Leutragraben',
            'City'    => 'Jena',
            'Zipcode' => '07740',
            'Company' => 'epages',
        }
    );

    return 1;
}

RunScript( Sub => \&Main, Debug => 1 );
