use Test::Simple tests => 25;
use strict;
use DE_EPAGES::Cartridge::API::Constants qw( STATE_INSTALLED );
use DE_EPAGES::Cartridge::API::Table::Cartridge qw( DeleteCartridge GetPKeyCartridgesByPackage InsertCartridge );
use DE_EPAGES::Core::API::Error qw (ExistsError GetError);
use DE_EPAGES::Core::API::Script qw (RunScript);
use DE_EPAGES::Database::API::Connection qw( ConnectTestSite DisconnectSite );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);

## no critic qw( ProhibitMagicNumbers )
sub Main {
    ConnectTestSite();

    Cleanup();

    my $System = LoadRootObject();
    my $Shop = $System->insertChild( 'Shop', { Alias => 'S1' } );
    ok( $Shop->id, 'create shop' );

    my $CartridgeID = InsertCartridge( { Package => 'C1::C1', Version => 25, InstallState => STATE_INSTALLED, DBStructureVersion => 25} );
    ok( $CartridgeID, 'InsertCartridge' );

    my $Features = $System->folder('Features');
    my $Feature1 = $Features->insertChild( 'Feature', {
        Alias => 'F1',
        CartridgeID => $CartridgeID,
        MaxValue => 1
    } );
    my $Feature2 = $Features->insertChild( 'Feature', {
        Alias => 'F2',
        CartridgeID => $CartridgeID,
        MaxValue => 1000
    } );
    ok( $Feature2->id, 'create feature' );
    ok( $Feature2->get('CartridgeID') == $CartridgeID, 'Feature.getCartridgeID' );
    ok( $Feature2->get('MaxValue') == 1000, 'Feature.getMaxValue' );

    # Feature Languages and Currencies always exist
    my $InitialFeatureCount = scalar @{$Shop->features};

    ok(($Shop->featureMaxValue('Currencies') >= 1), 'shop has at least one currency-feature' );
    ok(($Shop->featureMaxValue('Languages') >= 1), 'shop has at least one language-feature' );
    ok( 0 == $Shop->featureMaxValue( $Feature1 ), 'shop doesn\'t have feature 1' );

    $Shop->featureMaxValue( $Feature1, 1 );
    ok( $InitialFeatureCount +1 == scalar @{$Shop->features}, 'shop now has one feature' );
    ok( 1 == $Shop->featureMaxValue( $Feature1 ), 'featureMaxValue' );
    ok( 0 == $Shop->featureCurrentValue( $Feature1 ), 'featureCurrentValue 0' );
    ok( 1 == scalar(grep { $_->id == $Feature1->id } @{$Shop->features}), 'shop feature id' );

    $Shop->featureMaxValue( $Feature2, 100 );
    ok( $InitialFeatureCount + 2 == scalar @{$Shop->features}, 'shop now has two features' );
    ok( 100 == $Shop->featureMaxValue( $Feature2 ), 'featureMaxValue 100' );
    $Shop->featureCurrentValue( $Feature2, 20 );
    ok( 20 == $Shop->featureCurrentValue( $Feature2 ), 'featureCurrentValue 20' );

    $Shop->featureMaxValue( $Feature2, 0 );
    ok( 0 == $Shop->featureMaxValue( $Feature2 ), 'shop feature 2 removed' );

    $Shop->featureMaxValue( $Feature2, 10 );
    ok( $InitialFeatureCount + 2 == scalar @{$Shop->features}, 'shop feature 2 added again' );

    # test TLE hash access
    ok( 10 == $Shop->get('FeatureMaxValue')->{'F2'}, 'Shop.FeatureMaxValue.F2' );
    ok( 20 == $Shop->get('FeatureCurrentValue')->{'F2'}, 'Shop.FeatureCurrentValue.F2'  );

    # feature exceeded error
    eval { $Shop->featureCurrentValue( $Feature2, 30 ); };
    ok( ExistsError(), 'featureCurrentValue exceeds featureMaxValue' );
    ok( GetError()->code eq 'FeatureLimitExceeded', 'error code is FeatureLimitExceeded' );
    ok( GetError()->vars->{Feature} eq 'F2', 'error contains feature name' );
    ok( 20 == $Shop->featureCurrentValue( $Feature2 ), 'Feature value has not changed' );

    $Feature2->delete;
    ok( $InitialFeatureCount + 1 == scalar @{$Shop->features}, 'feature 2 deleted' );

    DeleteCartridge( $CartridgeID );
    ok( $InitialFeatureCount + 0 == scalar @{$Shop->features}, 'cartridge deleted' );

    Cleanup();
    DisconnectSite();
}

RunScript(
    Sub => \&Main
);

sub Cleanup {

    foreach my $Path (qw (/S1 /Features/F1 /Features/F2)) {
        if (ExistsObjectByPath($Path)) {
            my $object = LoadObjectByPath($Path);
            $object->delete;
        }
    }

    DeleteCartridge( $_ ) foreach @{GetPKeyCartridgesByPackage('C1::C1')};
}
