use strict;
use Test::Simple tests => 24;
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Object::API::XML::ObjectDeleteHandler qw( DeleteObjects );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::WebInterface::API::MessageCenter qw( SynchronizeCache );
use SOAP::Lite;

sub Main {

    my $hSystem;
    RunOnStore( Store => GetTestSite(), Sub => sub {
        $hSystem = LoadRootObject()->get([qw(DomainName WebServerDirectory WebServerScriptNamePart)]);
    });

    my $WebserviceURL = "http://%2FUsers%2FWSTestUser:WSTestUser\@$hSystem->{DomainName}/$hSystem->{WebServerDirectory}/$hSystem->{WebServerScriptNamePart}.soap";

    # create test classes
    my $FactoryService = SOAP::Lite
        ->uri('urn://epages.de/WebService/FactoryService/2004/08')
        ->proxy($WebserviceURL);

    my $ObjectService = SOAP::Lite
        ->uri('urn://epages.de/WebService/ObjectService/2004/08')
        ->proxy($WebserviceURL);

    # delete '/Test' if it exists
    if( $FactoryService->existsObjectByPath('/Test')->result ) {
        my $TestObjectID = $FactoryService->objectByPath('/Test')->result;
        $ObjectService->delete($TestObjectID);
    }

    # verify that '/Test' is deleted
    ok( !$FactoryService->existsObjectByPath('/Test')->result, "'/Test' does not exist" );

    # create '/Test' again
    my $RootObjectID = $FactoryService->rootObject->result;
    ok( $RootObjectID, "rootObject" );

    my $ClassID = $FactoryService->classByAlias('Object')->result;
    ok( $ClassID, "classByAlias" );

    my $ParentObjectID = $ObjectService->insertChild($RootObjectID, $ClassID, { Alias => 'Test'})->result;
    ok( $ParentObjectID, "insertChild" );

    # verify that '/Test' exists
    ok( $FactoryService->existsObjectByPath('/Test')->result, "'/Test' created" );
    ok( $ParentObjectID == $FactoryService->objectByPath('/Test')->result, "objectByPath" );

    my $ObjectID = $ObjectService->insertChild( $ParentObjectID, $ClassID, {
        Alias => 'MyObject',
    } )->result;
    ok( $ObjectID, "insertChild 2" );
    ok( $ObjectID == $FactoryService->objectByPath('/Test/MyObject')->result, "objectByPath 2" );

    # object id
    ok( $ObjectID > $ParentObjectID, "id is incremented" );
    ok( $ObjectID == $ObjectService->get( $ObjectID, ['ID'] )->result->{'ID'}, "get('ID')" );
    ok( $ObjectID == $ObjectService->get( $ObjectID, ['ObjectID'] )->result->{'ObjectID'}, "get('ObjectID')" );

    # class id
    ok( $ClassID == $ObjectService->get( $ObjectID, ['Class'] )->result->{'Class'}, "get('Class')" );

    # multiple attributes
    my $hAttributes = $ObjectService->get( $ObjectID, ['Inherit','Alias','GUID','Path'] )->result;
    ok( 4 == scalar keys %$hAttributes, "get multiple attributes" );

    # inherit
    ok( $hAttributes->{'Inherit'}, "Inherit" );

    # alias
    ok( $hAttributes->{'Alias'} eq 'MyObject', "Alias" );

    # guid
    ok( 36 == length( $hAttributes->{'GUID'} ) , "length(GUID)" );

    # path
    ok( $hAttributes->{'Path'} eq '/Test/MyObject', "Path" );

    # children
    my $aChildren = $ObjectService->get( $ObjectID, ['ChildObjects'] )->result->{'ChildObjects'};
    ok( 0 == scalar @$aChildren, "attribute ChildObjects" );

    # test children cache
    my $TestCacheObjectID = $ObjectService->insertChild($ObjectID, $ClassID, {'Alias'=>'TestCache'})->result;
    ok ($ObjectService->existsChild($ObjectID, 'TestCache')->result, 'object child exists');

    # change the parent form /Test/MyObject to /Test
    ok (! $ObjectService->existsChild($ParentObjectID, 'TestCache')->result, 'parent child doesnt exists');
    $ObjectService->set($TestCacheObjectID, { 'Parent' => $ParentObjectID });
    ok (! $ObjectService->existsChild($ObjectID,'TestCache')->result, 'object child doesnt exists');
    ok ($ObjectService->existsChild($ParentObjectID,'TestCache')->result, 'parent child exists');

    # change the alias
    $ObjectService->set($TestCacheObjectID, { 'Alias' => 'NewAlias' });
    ok (! $ObjectService->existsChild($ParentObjectID, 'TestCache')->result, 'changed alias doesnt exists');
    ok ($ObjectService->existsChild($ParentObjectID, 'NewAlias')->result, 'new alias exists');
}

sub Initial {
    RunOnStore( Store => GetTestSite(), Sub => sub {
        ImportObjects('Object.xml');
        SynchronizeCache();
    });
}

sub Cleanup {
    RunOnStore( Store => GetTestSite(), Sub => sub {
        DeleteObjects('Object.xml');
        SynchronizeCache();
    });
}

RunScript(Sub => sub {
    Cleanup();
    Initial();
    Main();
    Cleanup();
}, Debug=>1);
