#!/bin/env perl
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBOBGGRl5jMabHSgyARAqMvAJ9kseayWLBQdVgX3/ZfT89Z3QaK3QCg0u6C
#F6pFiqQsb4z/edxtCNF8Mvo=
#=x9Tk
#-----END PGP SIGNATURE-----
#line 2

use strict;
use Config;
use Getopt::Long;
use Crypt::GPG;
use DateTime;
use Cwd;
use File::Find;
use File::Basename qw ( dirname basename );
use File::Path qw ( mkpath );
use File::Temp qw ( tempfile );
use Config::IniFiles;
use LWP::Simple;
use Sys::Hostname;
use Time::HiRes qw( usleep );
use Data::Dumper;

use constant DECI_SECONDS => 100000; # sleep some time

our $BASENAME = basename($0);
$BASENAME =~ s/^(.+?)\..*/$1/;
our $DEF_INI = "$ENV{EPAGES_CONFIG}/$BASENAME.conf";
our $DEF_WRAP_SH = "$ENV{EPAGES}/bin/wrapScheduler.sh";
our $DEF_WRAP_PL = "$ENV{EPAGES}/bin/wrapScheduler.PL";

our $ADD = 1;
our $DEL = 2;
our $DETACHED = 1;
our $INTERNAL = 2;

our (%OPTIONS, $NOMINAL_STAT, $ACTUAL_STAT, $NOMINAL_ERROR, $ACTUAL_ERROR, @STAT_CHECK);
our ($GPG, $RESULT, $FILES, $SIG_INTERNAL, $VRF_PATH, $SIG_DETACHED, $SIG_PATH, $TRIES, $DELAY, $NEW_ERROR);

our %ERRORCODE = (
    'SUCCESS'          => 0,
    'USAGE_ERROR'      => 1,
    'READ_ERROR'       => 2,
    'WRITE_ERROR'      => 3,
    'VERIFY_FAILED'    => 4,
    'SIGN_FAILED'      => 5,
    'ADDKEY_FAILED'    => 6,
    'NOT_SIGNED'       => 8,
    'NO_SIG_FILE'      => 9,
    'ALREADY_SIGNED'   => 10,
    'ALREADY_SIG_FILE' => 11,
    'LWP_SIMPLE_FAILED' => 16,
    );

$RESULT = 'SUCCESS';

use vars qw(*name *dir *prune);
*name   = *File::Find::name;
*dir    = *File::Find::dir;

sub QuoteRegexp {
    my ($String) = @_;
    return $String unless $String;

    $String =~ s/^\s+//;
    $String =~ s/\s+$//;
    $String =~ s,\\,/,g if $Config{osname} eq 'MSWin32';
    $String =~ s/([(){^\$*+?.[])/\\$1/g;
    return $String;
}

sub AbsPath {
    my ($File) = @_;
    return $File unless -e $File;

    my $Dir = $File;
    $Dir = dirname($File) unless -d $File;

    my $WorkingDir = getcwd();
    chdir $Dir or die "$Dir: $!";
    $Dir = getcwd();
    chdir $WorkingDir;

    return -d $File ? $Dir : "$Dir/". basename($File);
}

sub GetSigFile {

    return if -d $_;
    return unless -s $_;

    my $SigType;
    $SigType = $INTERNAL if $SIG_INTERNAL and /$SIG_INTERNAL\Z/s;
    $SigType = $DETACHED if $SIG_DETACHED and /$SIG_DETACHED\Z/s;
    return unless $SigType;

    my $SigFile = $name;

    if ($SigType == $DETACHED and $SIG_PATH) {
        $SigFile =~ s/$VRF_PATH//;
        $SigFile = "$SIG_PATH$SigFile.asc";
    } elsif ($SigType == $DETACHED) {
        $SigFile = "$name.asc";
    }

    if ($SigType == $DETACHED and not  -f $SigFile) {
        $RESULT = 'NO_SIG_FILE';
        print "gpg: 0 $SigFile\n"; return;
    }
    return ($SigFile, $SigType);
}

sub GetContentSignature {
    my ($SigFile, $SigType) = @_;
    my ($Content, $Signature, $SIG_FILE, $SIGN);

    if (not open $SIG_FILE, '<', $SigFile) { ##no critic qw(RequireBriefOpen)
        $RESULT = 'READ_ERROR';
        $ACTUAL_ERROR->{$name} = $RESULT;
        return;
    }

    binmode($SIG_FILE);
    $Content = do { local $/ = undef; <$SIG_FILE> };
    close $SIG_FILE or $RESULT = 'CLOSE_ERROR';

    return ($Content, $Signature) if $SigType != $DETACHED;

    $Signature = $Content;

    if (not open $SIGN, '<', $name) { ##no critic qw(RequireBriefOpen)
        $RESULT = 'READ_ERROR';
        $ACTUAL_ERROR->{$name} = $RESULT;
        return;
    }

    binmode($SIGN);
    $Content = do { local $/ = undef; <$SIGN> };
    close $SIGN or $RESULT = 'CLOSE_ERROR';

    return ($Content, $Signature);
}

sub VerifySignature {
    my ($Content, $Signature, $SigFile, $SigType) = @_;

    if ($SigType != $DETACHED) {
        $Content =~ s/\r\n/\n/sg;
        $Content =~ s/\r$//s;
        if ($Content =~ /(.*)(.+\n#-----END PGP SIGNATURE-----\n)(?:#line [0-9]\n)/s) {
            $Content = $1;
            $Signature = $2;
        } else {
            $RESULT = 'NOT_SIGNED';
            $ACTUAL_ERROR->{$name} = $RESULT;
            return;
        }
        if ($Signature =~ s/(.+?\n)(#-----BEGIN PGP SIGNATURE-----\n)/$2/s) {
            $Content = "$1$Content";
        }
        $Signature =~ s/^#//mg;
    }
    my (undef, $sig) = $GPG->verify($Content, $Signature);
    if (!defined $sig) {
        $RESULT = 'VERIFY_FAILED';
        $ACTUAL_ERROR->{$name} = $RESULT;
    } elsif ($sig->validity() ne 'GOOD') {
        $RESULT = 'VERIFY_FAILED';
        $ACTUAL_ERROR->{$name} = $RESULT;
    }
}

sub CheckIntegrity {
    return unless -f $_;

    my ($SigFile, $SigType) = GetSigFile();
    delete $ACTUAL_ERROR->{$name};
    if ($SigFile) {
        for (my $Count = 0; $Count < $TRIES; $Count++) {
            delete $ACTUAL_ERROR->{$name};
            my ($Content, $Signature) = GetContentSignature($SigFile, $SigType);
            if (defined $ACTUAL_ERROR->{$name}) {
                usleep($DELAY);
                next;
            }
            VerifySignature($Content, $Signature, $SigFile, $SigType);
            if (defined $ACTUAL_ERROR->{$name}) {
                usleep($DELAY);
            } else {
                print "gpg: + $name\n" if $OPTIONS{debug} > 1;
                last;
            }
        }
    }
    if (defined $ACTUAL_ERROR->{$name}) {
        if (defined $OPTIONS{again} or
            $ACTUAL_ERROR->{$name} ne $NOMINAL_ERROR->{$name}) {
            $NEW_ERROR = 1;
            my $Char = $RESULT eq 'NOT_SIGNED' ? '0' : '-';
            print "gpg: $Char $name\n";
        }
        return;
    }

    return if /^\.nfs/s or not scalar @STAT_CHECK;
    return if $FILES and not /$FILES\Z/s;

    my $FirstTime = scalar keys %$NOMINAL_STAT ? 0 : 1;

    for (my $Count = 0; $Count < $TRIES; $Count++) {
        print "check ($Count): $name\n" if $OPTIONS{debug} > 1;
        my @Stat = stat $_;
        $ACTUAL_STAT->{$name} = join(' ', @Stat[@STAT_CHECK]);
        next if $FirstTime;

        if ($ACTUAL_STAT->{$name} ne $NOMINAL_STAT->{$name}) {
            $ACTUAL_ERROR->{$name} = $ACTUAL_STAT->{$name};
            usleep($DELAY);
        } else {
            delete $ACTUAL_ERROR->{$name};
            last;
        }
    }
    if (defined $ACTUAL_ERROR->{$name}) {
        if (defined $OPTIONS{again} or
            $ACTUAL_ERROR->{$name} ne $NOMINAL_ERROR->{$name}) {
            $NEW_ERROR = 1;
            print "stat: - $name\n";
        }
    }
}

sub AddPublicKeys {
    my ($PublicKeyList, $KnownPublicKeyFile, $GpgExe) = @_;
    return unless defined $PublicKeyList;

    my %KnownPublicKeys;

    if (-f $KnownPublicKeyFile) {
        open my $FOO, '<', $KnownPublicKeyFile or die "$KnownPublicKeyFile: $!";
        do { chomp($_); $KnownPublicKeys{$_} = 1;} foreach <$FOO>;
        close $FOO or $RESULT = 'CLOSE_ERROR';
    }

    my $NewPublicKey;
    my $Nul = $Config{osname} eq 'MSWin32' ? 'nul' : '/dev/null';
    my (undef, $TempFile) = tempfile(UNLINK => 1);

    foreach my $Url (split(/,/, $PublicKeyList)) {
        $Url =~ s/\$ENV{(.+?)}/$ENV{$1}/g;
        $Url =~ s,\\,/,g;
        if (-d $Url) {
            foreach my $File (glob("$Url/*")) {
                next if defined $KnownPublicKeys{$File};
                print "$GpgExe --import $File\n";
                # run it twice because 1st time directory .gnupg may not exist
                system("\"$GpgExe\" --import \"$File\" 2>$Nul");
                usleep(4 * DECI_SECONDS);
                print `"$GpgExe" --import "$File"`;
                if ( $? == 0 ) {
                    $NewPublicKey = 1;
                    $KnownPublicKeys{$File} = 1;
                } else {
                    $RESULT = 'ADDKEY_FAILED';
                }
            }
        } else {
            next if defined $KnownPublicKeys{$Url};
            my $File;
            if ($Url =~ m,^[a-z]+://.+,i) {
                $File = $TempFile;
                getstore($Url, $File) or $RESULT = 'LWP_SIMPLE_FAILED';
            } else {
                $File = $Url;
            }
            print "$GpgExe --import $File\n";
            # run it twice because 1st time directory .gnupg may not exist
            system("\"$GpgExe\" --import \"$File\" 2>$Nul");
            usleep(4 * DECI_SECONDS);
            print `"$GpgExe" --import "$File"`;
            if ( $? == 0 ) {
                $NewPublicKey = 1;
                $KnownPublicKeys{$Url} = 1;
            } else {
                $RESULT = 'ADDKEY_FAILED';
            }
        }
    }

    if ($NewPublicKey) {
        open my $FOO, '>', $KnownPublicKeyFile or die "$KnownPublicKeyFile: $!";
        print $FOO "$_\n" foreach keys %KnownPublicKeys;
        close $FOO or $RESULT = 'CLOSE_ERROR';
    }

    die $RESULT if $ERRORCODE{$RESULT};
}

sub GetDirectories {

    my $MyIniDir = AbsPath(dirname($OPTIONS{ini}));
    my $IniDir = AbsPath($ENV{EPAGES_CONFIG});
    ## no critic qw(RequireInterpolationOfMetachars)
    $IniDir = $IniDir eq $MyIniDir ? '$ENV{EPAGES_CONFIG}' : $MyIniDir;

    my $MyCmdDir = AbsPath(dirname($0));
    my $CmdDir = AbsPath("$ENV{EPAGES}/bin");
    $CmdDir = $CmdDir eq $MyCmdDir ? '$ENV{EPAGES}/bin' : $MyCmdDir;
    ## use critic

    return ($CmdDir, $IniDir);
}

sub RewriteIniFile {
    my ($IniFile, $Ini) = @_;

    my ($CmdDir, $IniDir) = GetDirectories();

    my $SchedulerConf = Config::IniFiles->new( -file => $IniFile ) or die "$IniFile: $!";

    foreach my $Section (keys %$Ini) {
        next if $OPTIONS{section} and $OPTIONS{section} ne $Section;
        next if $Section =~ /^:/;
        next unless $Ini->{$Section}{SubTasks};

        do {  $SchedulerConf->newval($Section, $_, $Ini->{$Section}{$_}) } foreach keys %{$Ini->{$Section}};

        my $Value = $Ini->{$Section}{Command};
        $Value = "$CmdDir/". basename($0) unless defined $Value;
        $SchedulerConf->newval($Section, 'Command', $Value);

        $Value = $Ini->{$Section}{Options};
        $Value = "-section $OPTIONS{section} -ini $IniDir/", basename($OPTIONS{ini}) unless defined $Value;
        $SchedulerConf->newval($Section, 'Options', $Value);
    }

    $SchedulerConf->RewriteConfig();
}

sub AddSchedulerItem {
    my ($Ini) = @_;
    RewriteIniFile("$ENV{EPAGES_CONFIG}/Scheduler.conf", $Ini);
}

sub DelSchedulerItem {
    my ($Ini) = @_;

    my $SchedulerConf = Config::IniFiles->new( -file => "$ENV{EPAGES_CONFIG}/Scheduler.conf" ) or die "$ENV{EPAGES_CONFIG}/Scheduler.conf: $!";

    my %Tasks;
    foreach my $Section (keys %$Ini) {
        next if $OPTIONS{section} and $OPTIONS{section} ne $Section;
        next if $Section =~ /^:/;
        next unless $Ini->{$Section}{SubTasks};
        $Tasks{$Section} = 1;
        $SchedulerConf->DeleteSection($Section);
    }
    $SchedulerConf->RewriteConfig();
}

sub RewriteCrontab {
    my ($Rewrite, $Ini) = @_;

    return unless $Rewrite;
    return if $Config{osname} eq 'MSWin32';

    my $HostName = $Ini->{':Default:'}{hostname};
    my $CronDir = "$ENV{HOME}/.epages/cron.d";
    mkpath($CronDir);

    foreach my $Section (keys %$Ini) {
        next if $OPTIONS{section} and $OPTIONS{section} ne $Section;
        next if $Section =~ /^:/;
        next unless $Ini->{$Section}{SubTasks};

        unlink("/etc/cron.d/$BASENAME,$Section") if $Rewrite & $DEL;
        unlink("$CronDir/$BASENAME,$Section") if $Rewrite & $DEL;

        next unless $Rewrite & $ADD;
        next if defined $Ini->{$Section}{IsActive} and not $Ini->{$Section}{IsActive};
        if ($Ini->{$Section}{Machine}) {
            next if $Ini->{$Section}{Machine} !~ /(?:^|,)$HostName(?:,|$)/;
        }

        my @RunAt = split(/\s+/, $Ini->{$Section}{Cron});
        next unless scalar @RunAt == 5;

        my $WrapSh = $Ini->{$Section}{"WrapSchedulerSh:$HostName"};
        $WrapSh = $Ini->{$Section}{WrapSchedulerSh} unless defined $WrapSh;
        $WrapSh = $DEF_WRAP_SH unless $WrapSh;
        $WrapSh =~ s/\$ENV{(.+?)}/$ENV{$1}/g;

        my $WrapPl = $Ini->{$Section}{"WrapSchedulerPl:$HostName"};
        $WrapPl = $Ini->{$Section}{WrapSchedulerPl} unless defined $WrapPl;
        $WrapPl = $DEF_WRAP_PL unless $WrapPl;
        $WrapPl =~ s/\$ENV{(.+?)}/$ENV{$1}/g;

        my $Project = $Ini->{$Section}{"UseBinEnvPerl:$HostName"};
        $Project = $Ini->{$Section}{UseBinEnvPerl} unless defined $Project;
        $Project = $ENV{EPAGES_PROJECT} unless $Project;

        open my $FILE, '>', "$CronDir/$BASENAME,$Section" or die "$CronDir/$BASENAME,$Section: $!";
        print $FILE join(' ', @RunAt, (getpwuid($<))[0]);
        print $FILE " \"$WrapSh\" -w \"$WrapPl\" -i \"$OPTIONS{ini}\" -g \"$BASENAME\" \"$Section\" \"own:$Project\"\n";
        close $FILE or $RESULT = 'CLOSE_ERROR';
    }
}

sub GetGpgPath {
    my $Delimiter = $Config{osname} eq 'MSWin32' ? ';' : ':';
    my @Ext = $Config{osname} eq 'MSWin32' ? ('.exe','2.exe') : ('','2');

    my %Devices;
    foreach my $Dir (split $Delimiter, $ENV{PATH}) {
        foreach( @Ext ) {
            return "gpg$_" if -x "$Dir\\gpg$_";
            return "gpg$_" if -f "$Dir\\gpg$_" and $Config{osname} eq 'MSWin32';
           }
        $Dir =~ /(.+?):/; $Devices{"\U$1\Q"} = 1;
    }
    return unless $Config{osname} eq 'MSWin32';

    foreach my $Drive (keys %Devices) {
        foreach my $Dir (glob("$Drive:\\*")) {
            foreach (qw(GNU GnuPG 1)) {
                return "$Dir\\gpg.exe" if -f "$Dir\\gpg.exe";
                return "$Dir\\gpg2.exe" if -f "$Dir\\gpg2.exe";
                $Dir .= "\\$_";
            }
        }
    }
}

sub ReadNominalStat {
    my ($List) = @_;

    $NOMINAL_STAT = {};
    if (-f "$List.NOMINAL") {
        do "$List.NOMINAL" or die $!;
    }

    $NOMINAL_ERROR = {};
    if (-f "$List.ERROR") {
        do "$List.ERROR" or die $!;
    }
}

sub DumpErrorStat {
    my ($List, $Header) = @_;

    if (not scalar keys %$ACTUAL_ERROR) {
        unlink "$List.ERROR";
        return;
    }

    $ACTUAL_ERROR->{$_} eq $NOMINAL_ERROR->{$_} or $NEW_ERROR = 1 foreach keys %$NOMINAL_ERROR;
    return unless $NEW_ERROR;

    open my $OUTPUT, '>', "$List.ERROR" or die "$List.ERROR: $!";
    print $OUTPUT "# fields in value: $Header\n";
    print $OUTPUT Data::Dumper->Dump([$ACTUAL_ERROR], [qw(NOMINAL_ERROR)]);
    print $OUTPUT "1;\n";
    close $OUTPUT or $RESULT = 'CLOSE_ERROR';

    my @ChangedFiles = keys %$ACTUAL_ERROR;
    my $dt = DateTime->now;
    my $DateTime = $dt->ymd. '+'. $dt->hms('-');
    $List =~ s,/,\\,g if $Config{osname} eq 'MSWin32';
    # tar is much faster and requires much less memory than Archive::Tar
    if ($Config{osname} eq 'MSWin32') {
        open my $TAR_GZ, '>', "$List.$DateTime.txt" or die "$List.$DateTime.txt: $!";
        s,/,\\,g foreach @ChangedFiles;
        print $TAR_GZ join("\0", @ChangedFiles);
        close $TAR_GZ or $RESULT = 'CLOSE_ERROR';
        # this is the only way to use -T in windows
        system("tar cf $List.$DateTime.tar --null -T $List.$DateTime.txt 2>nul");
        system("gzip -9 $List.$DateTime.tar");
    } else {
        open my $TAR_GZ, '|-', "tar cfT - - 2>/dev/null | gzip -c9 > $List.$DateTime.tar.gz" or die "$List.$DateTime.tar.gz: $!";
        print $TAR_GZ join("\n", @ChangedFiles);
        close $TAR_GZ or $RESULT = 'CLOSE_ERROR';
    }
    print "Changed files saved in:\n$List.$DateTime.tar.gz\n\n";
}

sub DumpNominalStat {
    my ($List, $Header) = @_;

    open my $OUTPUT, '>', $List or die "$List: $!";
    print $OUTPUT "# fields in value: $Header\n";
    print $OUTPUT Data::Dumper->Dump([$ACTUAL_STAT], [qw(NOMINAL_STAT)]);
    print $OUTPUT "1;\n";
    close $OUTPUT or $RESULT = 'CLOSE_ERROR';
}

sub SignatureCheck {
    my ($Ini) = @_;

    foreach my $Section (keys %$Ini) {

        next if $Section =~ /^:.*:$/;
        next if defined $OPTIONS{section} and $OPTIONS{section} ne $Section;
        next if not defined $OPTIONS{section} and not $Ini->{$Section}{SubTasks};
        next if $Config{osname} ne 'MSWin32' and $Ini->{$Section}{SubTasks} and not $Ini->{$Section}{Cron};
        next if $Config{osname} eq 'MSWin32' and $Ini->{$Section}{SubTasks} and not $Ini->{$Section}{Schtasks} and not $Ini->{$Section}{At};

        my @Tasks = ($Section);
        @Tasks = split(/,/, $Ini->{$Section}{SubTasks}) if $Ini->{$Section}{SubTasks};

        foreach my $Task (@Tasks) {
            next if defined $Ini->{$Task}{IsActive} and not $Ini->{$Task}{IsActive};
            next unless defined $Ini->{$Task}{Check};
            return 1 if $Ini->{$Task}{Check} =~ /signature/;
        }
    }
    return 0;
}

sub NewIntegrity {
    my ($Ini) = @_;

    $Ini->{':Default:'}{hostname} = hostname();
    chomp($Ini->{':Default:'}{hostname});
    $Ini->{':Default:'}{hostname} =~ s/\..*//;
    my $HostName = $Ini->{':Default:'}{hostname};

    my $VarDir = $Ini->{':Default:'}{"VarDir:$HostName"};
    $VarDir = $Ini->{':Default:'}{VarDir} unless defined $VarDir;
    if (!defined $VarDir || $VarDir !~ /\S/) {
        $VarDir = $Config{osname} eq 'MSWin32' ? $ENV{SYSTEMDRIVE} : $ENV{HOME};
        $VarDir .= '/.epages';
    }
    my $KnownPublicKeyFile = "$VarDir/$HostName.gpg";
    $VarDir .= "/$BASENAME";
    mkpath($VarDir);

    return "$VarDir/$HostName" unless SignatureCheck($Ini);

    my $GpgExe = $Ini->{':Default:'}{"GpgPath:$HostName"};
    $GpgExe = $Ini->{':Default:'}{GpgPath} unless defined $GpgExe;
    $GpgExe = GetGpgPath() unless $GpgExe;
    $GpgExe = 'gpg' unless $GpgExe;

    my $PublicKeys = $Ini->{':Default:'}{"PublicKeys:$HostName"};
    $PublicKeys = $Ini->{':Default:'}{PublicKeys} unless defined $PublicKeys;
    AddPublicKeys($PublicKeys, $KnownPublicKeyFile, $GpgExe);

    $GPG = Crypt::GPG->new;
    if (basename($GpgExe) ne $GpgExe) {
        $ENV{PATH} .= $Config{osname} eq 'MSWin32' ? ';' : ':';
        $ENV{PATH} .= dirname($GpgExe);
    }
    $GPG->gpgbin(basename($GpgExe));

    return "$VarDir/$HostName";
}

sub Main { ## no critic qw(ProhibitExcessComplexity)

    local $| = 1;

    my @GetOpt = qw(ini=s section=s again add:s del:s debug:i help);

    GetOptions(\%OPTIONS, @GetOpt) or usage();
    usage() if $OPTIONS{help};

    die "$0: EPAGES: no such environment variable\n" unless $ENV{EPAGES};

    $OPTIONS{debug} = 1 if defined $OPTIONS{debug} and not $OPTIONS{debug};
    $OPTIONS{debug} = 0 unless defined $OPTIONS{debug};
    $OPTIONS{ini} = $DEF_INI unless defined $OPTIONS{ini};

    my %Ini;
    tie %Ini, 'Config::IniFiles', ( -file => $OPTIONS{ini} ) or die "$OPTIONS{ini}: $!";

    if (defined $OPTIONS{del}) {
        DelSchedulerItem(\%Ini);
        RewriteCrontab($DEL, \%Ini);
        exit 0;
    }

    my $VarFile = NewIntegrity(\%Ini);

    ## no critic qw(ProhibitMagicNumbers)
    my %StatList = ('dev' => [0], 'inode' => [1], 'mode' => [2], 'nlink' => [3], 'uid' => [4],
                    'gid' => [5], 'owner' => [4,5], 'rdev' => [6], 'size' => [7], 'atime' => [8],
                    'mtime' => [9], 'ctime' => [10], 'blksize' => [11], 'blocks' => [12],);
    # use critic

    my %ListedFiles;

    foreach my $Section (keys %Ini) {

        next if $Section =~ /^:.*:$/;
        next if defined $OPTIONS{section} and $OPTIONS{section} ne $Section;
        next if not defined $OPTIONS{section} and not $Ini{$Section}{SubTasks};
        next if $Config{osname} ne 'MSWin32' and $Ini{$Section}{SubTasks} and not $Ini{$Section}{Cron};
        next if $Config{osname} eq 'MSWin32' and $Ini{$Section}{SubTasks} and not $Ini{$Section}{Schtasks} and not $Ini{$Section}{At};

        my @Tasks = ($Section);
        @Tasks = split(/,/, $Ini{$Section}{SubTasks}) if $Ini{$Section}{SubTasks};

        foreach my $Task (@Tasks) {

            next if defined $Ini{$Task}{IsActive} and not $Ini{$Task}{IsActive};
            next unless defined $Ini{$Task}{Check};
            next unless $Ini{$Task}{Check} =~ /\S/;
            next if defined $ListedFiles{$Task};

            $ListedFiles{$Task} = 1;

            my $dt = DateTime->now;
            print $dt->ymd, '+', $dt->hms('-'), "\n" if $OPTIONS{debug};
            print "Task $Task: $Ini{$Task}{Check}\n" if $OPTIONS{debug};

            $ACTUAL_STAT = {};
            $ACTUAL_ERROR = {};
            $NEW_ERROR = undef;

            @STAT_CHECK = map { @{$StatList{$_}} } grep { defined $StatList{$_} } split /,/, $Ini{$Task}{Check};

            my $FileListFile = "$VarFile.$Task";
            my $Add = $OPTIONS{add};
            $Add = 1 unless -f "$FileListFile.NOMINAL";
            unlink "$FileListFile.NOMINAL", "$FileListFile.ERROR" if $OPTIONS{add};

            ReadNominalStat($FileListFile);

            print "=> $Section->$Task->$Ini{$Task}{CheckPath}: stat ", join(',', @STAT_CHECK), "\n"
                if $OPTIONS{debug} and scalar @STAT_CHECK;

            $FILES = QuoteRegexp($Ini{$Task}{CheckFiles});

            ($SIG_INTERNAL, $SIG_DETACHED) = (undef, undef);

            if ($Ini{$Task}{Check} =~ /signature/) {
                $SIG_INTERNAL = QuoteRegexp($Ini{$Task}{InternalSignatures});
                $SIG_DETACHED = QuoteRegexp($Ini{$Task}{ExternalSignatures});
                $SIG_PATH = $Ini{$Task}{SignaturePath};
                $SIG_PATH =~ s/\$ENV{(.+?)}/$ENV{$1}/g;
                $SIG_PATH =~ s,\\,/,g;
                $SIG_PATH = AbsPath($SIG_PATH);
                print "=> $Section->$Task->$Ini{$Task}{CheckPath}: verify signature\n" if $OPTIONS{debug};
            }

            my @Directories;
            foreach $VRF_PATH (split /,/, $Ini{$Task}{CheckPath}) {
                $VRF_PATH =~ s/\$ENV{(.+?)}/$ENV{$1}/g;
                $VRF_PATH =~ s,\\,/,g;
                $VRF_PATH = AbsPath($VRF_PATH);
                push(@Directories, $VRF_PATH) if -d $VRF_PATH;
            }
            $TRIES = defined $Ini{$Task}{Tries} ? $Ini{$Task}{Tries} : $Ini{':Default:'}{Tries};
            $TRIES = 1 unless defined $TRIES;
            $TRIES = 1 unless $TRIES =~ /^[0-9]+$/;
            $DELAY = defined $Ini{$Task}{Delay} ? $Ini{$Task}{Delay} : $Ini{':Default:'}{Delay};
            $DELAY = DECI_SECONDS unless defined $DELAY;
            $DELAY = DECI_SECONDS unless $DELAY =~ /^[0-9]+$/;
            File::Find::find({ wanted => \&CheckIntegrity, follow_fast => 1 }, @Directories);

            my $Header = $Ini{$Task}{Check};
            $Header =~ s/,/ /g;
            $Header =~ s/owner/uid gid/;

            DumpErrorStat($FileListFile, $Header);
            DumpNominalStat("$FileListFile.NOMINAL", $Header) if $Add and scalar @STAT_CHECK;
        }
    }
    my $dt = DateTime->now;
    print $dt->ymd, '+', $dt->hms('-'), "\n" if $OPTIONS{debug};

    if ($OPTIONS{add}) {
        exit 0 if $OPTIONS{add} eq '';
        my ($Var, $Val) = split(/=/, $OPTIONS{add});
        usage() unless $Var eq 'scheduler';

        $Val = '' unless defined $Val;
        if ($Val eq '') {
            $Val = $Config{osname} eq 'MSWin32' ? 'epages' : 'cron';
        }
        usage() if $Config{osname} eq 'MSWin32' and $Val eq 'cron';

        if ($Val eq 'epages') { AddSchedulerItem(\%Ini); }
        elsif ($Val eq 'cron') { RewriteCrontab($ADD, \%Ini); }
        else { usage(); }
    }
    exit $ERRORCODE{$RESULT};
}

sub usage {
    print <<END_USAGE;

Description:
    Check the integrity of the ePages file system

Usage:
    perl $0 [Options]

Options:
    -help          show the command line options
    -ini file      read configuration file (default: $DEF_INI)
    -section name  section in ini file to check (default: any)
    -again         show output even if result is same as in run before
    -add [scheduler[=SCHED]] initialize check; if argument exists:
                     o  it may have following values:
                        epages: add the check to epages schedulers
                        cron: add the check to the crontab (only UNIX)
                     o  no value defaults to epages (Windows) resp. cron (UNIX)
    -del           remove check from epages scheduler and crontab

Example:
    perl $0
END_USAGE
    exit 2;
}

Main();
1;
