#!/bin/env perl
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBN8dq3l5jMabHSgyARApGsAJ9AmBO1a7OfUQugd5mDYRyLSnUYjQCgysrl
#xOUDNWu6n+8rICgFNLGZOGw=
#=Poj7
#-----END PGP SIGNATURE-----
#line 2

use strict;
use Config;
use Getopt::Long;
use Crypt::GPG;
use Cwd;
use File::Find;
use File::Basename qw ( dirname basename );
use File::Path qw ( mkpath );

our ($GPG, %OPTIONS, $RESULT);

our $VALID_FILE_TYPE = 'PL|cfg|conf|config|mak|pl|pm|sh|t';

our %ERRORCODE = (
    'SUCCESS'          => 0,
    'USAGE_ERROR'      => 1,
    'READ_ERROR'       => 2,
    'WRITE_ERROR'      => 3,
    'VERIFY_FAILED'    => 4,
    'SIGN_FAILED'      => 5,
    'NOT_SIGNED'       => 8,
    'NO_SIG_FILE'      => 9,
    'ALREADY_SIGNED'   => 10,
    'ALREADY_SIG_FILE' => 11,
    'WRONG_FILE_TYPE'  => 12,
    );

$RESULT = 'SUCCESS';

use vars qw(*name *dir *prune);
*name   = *File::Find::name;
*dir    = *File::Find::dir;

sub QuoteRegexp {
    my ($String) = @_;
    $String =~ s/^\s+//;
    $String =~ s/\s+$//;
    $String =~ s,\\,/,g if $Config{osname} eq 'MSWin32';
    $String =~ s/([(){^\$*+?.[])/\\$1/gx;
    return $String;
}

sub GetSigFile {
    my $SigFile = $name;

    return if -d $_;
    return unless -s $_;
    return if defined $OPTIONS{files} && !/($OPTIONS{files})\Z/s;

    if ($OPTIONS{detach}) {
        my $Dir = $OPTIONS{is_file} ? $OPTIONS{parent} : $OPTIONS{dir};
        $SigFile =~ s/$Dir//;
        $SigFile = "$OPTIONS{detach}$SigFile.asc";
    } elsif (defined $OPTIONS{detach}) {
        $SigFile = "$name.asc";
    }

    if (((defined $OPTIONS{verify} && !-f $SigFile) ||
        (!defined $OPTIONS{verify} && -f $SigFile))
        && defined $OPTIONS{detach} && !defined $OPTIONS{resign} ) {
        $RESULT = -f $SigFile ? 'ALREADY_SIG_FILE' : 'NO_SIG_FILE';
        print "gpg: 0 $SigFile\n" if $OPTIONS{quiet} <= 1;
        return;
    }
    return $SigFile;
}

sub GetContentSignature {
    my ($SigFile) = @_;

    ## no critic qw(RequireBriefOpen RequireCheckedClose)
    my $Signature;
    my $FOO;
    if (defined $OPTIONS{verify})  {
        if (!open $FOO, '<', $SigFile) {
            $RESULT = 'READ_ERROR';
            print "gpg: - $SigFile\n" if $OPTIONS{quiet} <= 1;
            return;
        }
        if (defined $OPTIONS{detach}) {
            binmode($FOO);
            $Signature = do { local $/ = undef; <$FOO> };
            close $FOO;
            if (!open $FOO, '<', $name) {
                $RESULT = 'READ_ERROR';
                print "gpg: - $name\n" if $OPTIONS{quiet} <= 1;
                return;
            }
        }
    }
    elsif (!open $FOO, '<', $name) {
        $RESULT = 'READ_ERROR';
        print "gpg: - $name\n" if $OPTIONS{quiet} <= 1;
        return;
    }

    binmode($FOO);
    my $Content =  do { local $/ = undef; <$FOO> };
    close $FOO;
    return ($Content, $Signature);
    ## use critic
}

sub VerifySignature {
    my ($Content, $Signature, $SigFile) = @_;

    if (not defined $OPTIONS{detach}) {
        $Content =~ s/\r\n/\n/sg;
        $Content =~ s/\r$//s;
        if ($Content !~ /(.+\n\#-----END PGP SIGNATURE-----\n)(\#line [0-9]\n)/sx) {
            $RESULT = 'NOT_SIGNED';
            print "gpg: 0 $SigFile\n" if $OPTIONS{quiet} <= 1;
            return;
        }
        $Signature = $1; ## no critic qw(ProhibitCaptureWithoutTest)
        $Content = $'; ## no critic qw(ProhibitMatchVars) #'
        if ($Signature =~ s/(.+?\n)(\#-----BEGIN PGP SIGNATURE-----\n)/$2/sx) {
            $Content = "$1$Content";
        }
        $Signature =~ s/^#//mg;
    }
    my (undef, $sig) = $GPG->verify($Content, $Signature);
    if (!defined $sig) {
        $RESULT = 'VERIFY_FAILED';
        print "gpg: - $SigFile\n";
    } elsif ($sig->validity() eq 'GOOD') {
        print "gpg: + $SigFile\n" unless $OPTIONS{quiet};
    } else {
        $RESULT = 'VERIFY_FAILED';
        print "gpg: - $SigFile\n" if $OPTIONS{quiet} <= 1;
    }
}

sub SignFile {
    my ($Content, $SigFile) = @_;

    my $Document = $Content;
    my $DosLines = 0;

    if (!defined $OPTIONS{detach}) {
        $DosLines = $Document =~ s/\r\n/\n/sg ? 1 : 0;
        $Document =~ s/\r$//s;
        if ($Document =~ /^\#-----END PGP SIGNATURE-----$/mx) {
            $RESULT = 'ALREADY_SIGNED';
            print "gpg: 0 $SigFile\n" if $OPTIONS{quiet} <= 1;
            return;
        }
        elsif ($SigFile !~ /\.($VALID_FILE_TYPE)\Z/ && $Document !~ /^\#/s || -B $SigFile ) {
            $RESULT = 'WRONG_FILE_TYPE';
            print "gpg: # $SigFile\n" if $OPTIONS{quiet} <= 1;
            return;
        }
    }

    mkpath(dirname($SigFile)) unless -d dirname($SigFile);
    my $Signature = $GPG->sign($Document);
    if ($Signature) {
        if (!defined $OPTIONS{detach}) {
            $Signature .= "\n" unless $Signature =~ /\n$/s;
            my $LineNo = $Document =~ /^use Filter::netcrypt;/m ? 0 : 1;
            my $SheBang = $Document =~ /^\#(\!.*)/m;
            $LineNo++ if $SheBang and $LineNo;
            $Signature .= "line $LineNo\n";
            $Signature = "$1\n$Signature" if $SheBang;
            $Signature =~ s/^/#/mg;
            $Signature =~ s/\n/\r\n/sg if $DosLines;
            $Content =~ /.+\n/m;
            $Signature .= $SheBang ? $' : $Content; ## no critic qw(ProhibitMatchVars) #'
        }
        print"gpg: + $SigFile\n" unless $OPTIONS{quiet};
        open my $FOO, '>', $SigFile or do {
            $RESULT = 'WRITE_ERROR';
            return;
        };
        binmode($FOO);
        print $FOO $Signature;
        close $FOO; ## no critic qw(RequireCheckedClose)
    } else {
        $RESULT = 'SIGN_FAILED';
        print "gpg: - $SigFile\n" if $OPTIONS{quiet} <= 1;
    }
}

sub UnsignFile {
    my ($Content, $SigFile) = @_;

    return $Content unless -f $SigFile;

    if (defined $OPTIONS{detach}) {
        unlink($SigFile);
        print "gpg: u $SigFile\n" unless $OPTIONS{quiet};
        return $Content;
    }

    return $Content unless $Content =~ /(.+\n\#-----END PGP SIGNATURE-----\r?\n)/sx;

    my $Signature = $1; ## no critic qw(ProhibitCaptureWithoutTest)
    my $UnsignedContent = $'; ## no critic qw(ProhibitMatchVars) #'
    $UnsignedContent = "$1$UnsignedContent" if $Signature =~ /(.+?\n)\#-----BEGIN PGP SIGNATURE-----\r?\n/sx;

    open my $FOO, '>', $SigFile or do {
        $RESULT = 'WRITE_ERROR';
        print "gpg: U $SigFile\n" if $OPTIONS{quiet} <= 1;
        return $Content;
    };
    binmode($FOO);
    print $FOO $UnsignedContent;
    close $FOO; ## no critic qw(RequireCheckedClose)
    print "gpg: u $SigFile\n" unless $OPTIONS{quiet};

    return $UnsignedContent;
}

sub Signature {

    my $SigFile = GetSigFile() or return;
    my ($Content, $Signature) = GetContentSignature($SigFile);
    $Content or return;

    if (defined $OPTIONS{verify}) {
        VerifySignature($Content, $Signature, $SigFile);
    } else {
        $Content = UnsignFile($Content, $SigFile) if defined $OPTIONS{unsign};
        SignFile($Content, $SigFile) if defined $OPTIONS{sign};
    }
}

sub GetGpgPath {
    my $Delimiter = $Config{osname} eq 'MSWin32' ? ';' : ':';
    my @Ext = $Config{osname} eq 'MSWin32' ? ('.exe','2.exe') : ('','2');

    my %Devices;
    foreach my $Dir (split($Delimiter, $ENV{PATH})) {
        foreach (@Ext) {
            return "gpg$_" if -x "$Dir\\gpg$_";
            return "gpg$_" if -f "$Dir\\gpg$_" and $Config{osname} eq 'MSWin32';
        }
        $Dir =~ /(.+?):/;
        $Devices{"\U$1\Q"} = 1;
    }
    return unless $Config{osname} eq 'MSWin32';

    foreach my $Drive (keys %Devices) {
        foreach my $Dir (glob "$Drive:\\*") {
            foreach (qw(GNU GnuPG 1)) {
                return "$Dir\\gpg.exe" if -f "$Dir\\gpg.exe";
                return "$Dir\\gpg2.exe" if -f "$Dir\\gpg2.exe";
                $Dir .= "\\$_";
            }
        }
    }
}

sub Main {

    local $| = 1;

    my @GetOpt = qw(gpg=s key=s pass=s verify files=s dir=s detach:s quiet:i sign unsign resign help);

    GetOptions(\%OPTIONS, @GetOpt) or usage();
    usage() if $OPTIONS{help};
    usage() unless $OPTIONS{dir};

    die "$0: EPAGES: no such environment variable\n" unless $ENV{EPAGES};

    $OPTIONS{gpg} = GetGpgPath() unless $OPTIONS{gpg};
    $OPTIONS{gpg} = 'gpg' unless $OPTIONS{gpg};

    $OPTIONS{files} = $OPTIONS{files} ? QuoteRegexp($OPTIONS{files}) : undef;
    $OPTIONS{quiet} = 1 if defined $OPTIONS{quiet} && !$OPTIONS{quiet};

    $OPTIONS{sign} = $OPTIONS{resign} unless defined $OPTIONS{sign};
    $OPTIONS{sign} = 1 unless $OPTIONS{unsign};
    $OPTIONS{unsign} = 1 if defined $OPTIONS{resign};

    my $WorkingDir = getcwd();
    $OPTIONS{is_file} = -f $OPTIONS{dir} ? basename($OPTIONS{dir}) : undef;
    $OPTIONS{parent} = $OPTIONS{is_file} ? dirname($OPTIONS{dir}) : $OPTIONS{dir};
    chdir $OPTIONS{parent} or die "$OPTIONS{parent}: $!";
    $OPTIONS{dir} = getcwd();
    $OPTIONS{dir} .= "/$OPTIONS{is_file}" if $OPTIONS{is_file};
    chdir $WorkingDir;

    if ($OPTIONS{detach}) {
        mkpath($OPTIONS{detach}) unless -d $OPTIONS{detach};
        chdir $OPTIONS{detach} or die "$OPTIONS{detach}: $!";
        $OPTIONS{detach} = getcwd();
    }
    chdir $WorkingDir;

    my $DefaultKey = $OPTIONS{key};
    my $ListKeys = `"$OPTIONS{gpg}" --list-keys`;
    $ListKeys =~ s/\r?\nuid//sg;

    my %GpgKeys;
    foreach (split /\n/, $ListKeys) {
        if( /^pub\s+.*?\/(\S+)\s+(.*)/x ) {
            $GpgKeys{$1} = "$2";
            $DefaultKey = $1 unless $DefaultKey;
        }
    }
    die "$0: no gpg keys found\n" unless scalar keys %GpgKeys;

    if ($OPTIONS{key}) {
        my $GpgKeyFound = 0;
        $GpgKeyFound = 1 if grep { $OPTIONS{key} eq $_ } keys %GpgKeys;
        if (not $GpgKeyFound) {
            print "$0: void gpg key in option -key: $OPTIONS{key}\n";
            print "  Use one of (or omit option -key):\n";
            print "  $_: $GpgKeys{$_}\n" foreach keys %GpgKeys;
            usage();
        }
    }

    $GPG = Crypt::GPG->new;
    if (basename($OPTIONS{gpg}) ne $OPTIONS{gpg}) {
        $ENV{PATH} .= $Config{osname} eq 'MSWin32' ? ';' : ':';
        $ENV{PATH} .= dirname($OPTIONS{gpg});
    }
    $GPG->gpgbin(basename($OPTIONS{gpg}));
    if (not defined $OPTIONS{verify}) {
        $GPG->comment($GpgKeys{$DefaultKey});
        $GPG->secretkey("0x$OPTIONS{key}") if $OPTIONS{key};
        $GPG->passphrase($OPTIONS{pass}) if defined $OPTIONS{pass};
    }

    File::Find::find( {wanted => \&Signature, follow_fast => 1}, ($OPTIONS{dir}) );
    print "$0 (partially) failed: $RESULT\n" if $ERRORCODE{$RESULT};
    exit $ERRORCODE{$RESULT};
}

sub usage {
    print <<END_USAGE;

Description:
    Sign/verify files in ePages file system

Usage:
    perl $0 [Options]

Options:
    -help         show the command line options
    -gpg EXE      path to gpg executable (if in unusual place)
    -key KEY      8-digit hexadecimal number (if default is wrong)
    -pass "PASS"  pass phrase for key (if necessary)
    -verify       do not sign, verify files instead
    -unsign       do not sign, remove signature instead
    -resign       remove signature (if exist) and sign again
    -files "LIST" handle only files ending in LIST (separated by |)
    -dir DIR      handle files in DIR (the only required option)
    -detach DIR   use DIR for detached signatures
    -quiet        quiet operation
    -quiet 2      very quiet operation

Example:
    perl $0 -dir \$EPAGES_CARTRIDGES/myfile.pl
    perl $0 -files ".pl|.pm" -dir \$EPAGES_CARTRIDGES -key B2D28320
    perl $0 -files ".pl|.pm" -dir \$EPAGES_CARTRIDGES/COMPANY
    perl $0 -verify -dir \$EPAGES_PERL/bin -detach \$EPAGES_PERL/.gpg/bin
    perl $0 -key B2D28320 -resign -dir bin -files ".PL|.pl|.sh"
END_USAGE
    exit $ERRORCODE{USAGE_ERROR};
}

Main();
1;
