#!/bin/env perl
#-----BEGIN PGP SIGNATURE-----
#Version: GnuPG v1.2.6 (GNU/Linux)
#Comment: 2008-11-24 ePages Software GmbH (release key) <rnd@epages.de>
#
#iD8DBQBOKCmCl5jMabHSgyARAsILAKCQZi9qcV0si1mX44oKO7UbfJusJwCgtN2T
#vo28ExXHpnYgALTgmaFopQs=
#=2ENr
#-----END PGP SIGNATURE-----

use strict;

use Config;
use Getopt::Long;
use Sys::Hostname;
use Config::IniFiles;
use File::Path qw ( mkpath );
use File::Temp;
use Data::Dumper;

my $UseEpagesPerl;

BEGIN {
    use Config;
    ## no critic qw( ProhibitStringyEval )
    if ( $Config{osname} eq 'MSWin32' ) {
        eval 'use Win32::Process::Info'; die $@ if $@;
    }
    else {
        eval 'use Proc::ProcessTable'; die $@ if $@;
    }
    eval 'use DE_EPAGES::Core::API::Script qw( RunScript );';
    $UseEpagesPerl = $@ ? 0 : 1;
    eval 'use DE_EPAGES::MailType::API::TextMail qw( SendTextMail );'
        if $UseEpagesPerl;
    ## use critic
}

## no critic qw( RequireCorrectImports RequireCommentForExportedFunctions ProhibitExcessComplexity RequireBriefOpen )

my $CurrentTime = time();
my $DefaultIni = "$ENV{'EPAGES_CONFIG'}/Scheduler.conf";
my $MAIL_HEADER = 1;
my $MAIL_FOOTER = 2;
my $MAIL_FILE   = 4;

sub usage {
    print <<END_USAGE;
Usage:
    perl $0 [Options] SECTION

Options:
    -help    show the command line options
    -ini     read configuration file (default: $DefaultIni)
    -var     directory for variable data (default: VarDir in -ini)
    -maxage AGE  max job duration without a next instance reports an error
                 AGE is one of (default: MaxDuration in -ini):
                 <MINUTES> (e.g. -d 56)
                 <HOURS>:<MINUTES> (e.g. -d 7:30)
                 <DAYS>-<HOURS>:<MINUTES> (e.g. -d 3-00:00)
    SECTION  section in configuration file

Example:
    perl $0
END_USAGE
;
    exit 2;
}

#========================================================================================
# function : debug
#========================================================================================

sub debug {
    my ($LineNo, $Text) = @_;
    return unless $ENV{DEBUG_CRON};

    open my $DebugFH, '>>', $ENV{DEBUG_CRON} or die "$ENV{DEBUG_CRON}: $!";
    if ($LineNo) {
        $LineNo = sprintf '%3d', $LineNo;
        print $DebugFH "wrapScheduler.PL:$LineNo  // $Text\n";
    } elsif ( -f $Text ) {
        print $DebugFH "< $Text >\n";
        if ( -s $Text) {
            open my $ReportFH, '<', $Text or die "$Text: $!";
            my @Report = <$ReportFH>;
            print $DebugFH @Report;
            close $ReportFH or die "$Text: $!";
        }
        print $DebugFH "</ $Text >\n\n";
    } else {
        print $DebugFH "$Text\n";
    }
    close $DebugFH or die "$ENV{DEBUG_CRON}: $!";
}

#========================================================================================
# function : SendSmtpMail
#========================================================================================

sub SendSmtpMail {
    my ($Subject, $MailFile, $Options, $Ini) = @_;

    my $MailServer = $Ini->{$Options->{section}}{"MailConnection:$Options->{hostname}"};
    $MailServer = $Ini->{$Options->{section}}{MailConnection} unless defined $MailServer;
    my ($MailLogin, $MailPasswd);

    if ($MailServer) {
        if( $MailServer =~ s/(.+)\@// ) {
            $MailLogin = $1
        }
        if( $MailLogin =~ s/(.+):// ) {
            $MailPasswd = $1
        }
    } else {
        $MailServer = 'localhost';
    }

    my $From = $Ini->{$Options->{section}}{"MailSender:$Options->{hostname}"};
    $From = $Ini->{$Options->{section}}{MailSender} unless defined $From;
    my $To = $Ini->{$Options->{section}}{"MailRecipient:$Options->{hostname}"};
    $To = $Ini->{$Options->{section}}{MailRecipient} unless defined $To;

    $From = $MailLogin. '@'. $MailServer if not $From and $MailLogin;
    $From = (getpwuid($>))[0]. '@'. $MailServer if not $From and $Config{osname} ne 'MSWin32';
    $To = $From unless $To;

    my $MailHeader = "From: $From\nTo: $To\nSubject: $Subject\n";

    my $FileTest = -r $MailFile ? 1 : 0;
    my $Error = $FileTest ? '' : '>>> ERROR <<<: ';
    debug(__LINE__, "${Error}SendSmtpMail: MailFile == $MailFile (readable: $FileTest)");

    my $Content;
    {
        local $/ = undef;
        open my $FOO, '<', $MailFile or die "Can't open $MailFile: $!";
        $Content = <$FOO>;
        close $FOO or die "Can't close $MailFile: $!";;
    }

    my $Auth;
    $Auth = ", MailLogin=$MailLogin, MailPasswd=$MailPasswd" if $MailLogin;
    debug(__LINE__, "SendSmtpMail: MailServer=$MailServer$Auth");
    debug(__LINE__, "SendSmtpMail: From=$From, To=$To, Subject=$Subject\n");

    my $Smtp = Net::SMTP->new($MailServer);
    $Smtp->auth($MailLogin, $MailPasswd) if $MailLogin;
    foreach (split/[,;]/, $To) {
        $Smtp->mail($From); $Smtp->to($_);
        $Smtp->data(); $Smtp->datasend($MailHeader);
        $Smtp->datasend("$Content\n");
        $Smtp->dataend();
    }
    $Smtp->quit;
}

#========================================================================================
# function : MailHeader
#========================================================================================

sub MailHeader {
    my ( $Options, $Ini ) = @_;

    my $Date = scalar localtime($CurrentTime);
    my $TmpFile = TmpFileName($Options, '.run');
    my ($PerlExe, $UserName) = ProcessInfo();
    my $Script = $Ini->{$Options->{section}}{Command};
    my $Args = $Ini->{$Options->{section}}{Options};

    my $LoopString;
    if ($Ini->{$Options->{section}}{Loop}) {
        my $Loop = $Ini->{$Options->{section}}{Loop};
        $Loop = join("\n  ", split(/,/, $Ini->{LOOPS}{$Loop}));
        $LoopString =  "\nLOOP: $Ini->{$Options->{section}}{Loop}=$Loop" if ($Loop =~ /\S/);
    }

    return <<EOF;
*** Output from program $Options->{section} ***
COMMAND: $PerlExe $Script $Args > $TmpFile 2>&1$LoopString
HOSTNAME: $Options->{hostname}
USERNAME: $UserName
DATE: $Date

EOF
;
}

#========================================================================================
# function : MailFooter
#========================================================================================

sub MailFooter {
    my $Date = scalar localtime($CurrentTime);
    return "\nENDDATE:   ". localtime(time). "$Date\n\n";
}

#========================================================================================
# function : TmpFileName
#========================================================================================

sub TmpFileName {
    my ($Options, $Suffix) = @_;

    return "$Options->{tmp}/$Options->{hostname}.$Options->{section}$Suffix";
}

#========================================================================================
# function : VarFileName
#========================================================================================

sub VarFileName {
    my ($Options, $Suffix) = @_;

    return "$Options->{var}/$Options->{hostname}.$Options->{section}$Suffix";
}

#========================================================================================
# function : ReportError
#========================================================================================

sub ReportError {
    my ($Envelope, $ErrorMsg, $Options, $Ini) = @_;

    my $UseBinEnvPerl = $Ini->{$Options->{section}}{"UseBinEnvPerl:$Options->{hostname}"};
    $UseBinEnvPerl = $Ini->{$Options->{section}}{UseBinEnvPerl} unless defined $UseEpagesPerl;
    $UseBinEnvPerl = 1 unless $UseEpagesPerl;

    my $FileSize = 0;
    my $LogFile = VarFileName($Options, '.log');
    my $TmpFileBase = TmpFileName($Options);

    my $FileTest = 1;
    my ($LOG, $MAIL);
    if (not open $LOG, '>>', $LogFile) {
        warn "$LogFile: $!";
        $FileTest = 0;
    }
    my $Error = $FileTest ? '' : '>>> ERROR <<<: ';
    debug(__LINE__, "${Error}ReportError: LogFile == $LogFile (writeable: $FileTest)");

    $FileTest = 1;
    $FileTest = 0 unless open $MAIL, '>', "$TmpFileBase.mail";
    $Error = $FileTest ? '' : '>>> ERROR <<<: ';
    debug(__LINE__, "${Error}ReportError: TmpFileBase.mail == $TmpFileBase.mail (writeable: $FileTest)");
    die "$TmpFileBase.mail: $!" unless $FileTest;

    print $LOG MailHeader($Options, $Ini) if $Envelope & $MAIL_HEADER;
    print $MAIL MailHeader($Options, $Ini) if $Envelope & $MAIL_HEADER;
    print $LOG $ErrorMsg;
    print $MAIL $ErrorMsg;
    if ($Envelope & $MAIL_FILE and -s "$TmpFileBase.run") {
        $FileTest = -r "$TmpFileBase.run" ? 1 : 0;
        $Error = $FileTest ? '' : '>>> ERROR <<<: ';
        debug(__LINE__, "${Error}ReportError: TmpFileBase.run == $TmpFileBase.run (readable: $FileTest)");
        $FileSize = (stat("$TmpFileBase.run"))[7];
        open my $RUN, '<', "$TmpFileBase.run" or die "$TmpFileBase.run: $!";
        while (<$RUN>) {
            print $LOG $_;
            if ($FileSize <= 50000 or $UseBinEnvPerl) { ## no critic qw( ProhibitMagicNumbers )
                print $MAIL $_;
            }
        }
        close $RUN or warn "Can't close $TmpFileBase.run: $!";
    }
    print $LOG MailFooter() if $Envelope & $MAIL_FOOTER;
    close $LOG or warn "Can't close $LogFile: $!";
    close $MAIL or warn "Can't close $TmpFileBase.mail: $!";

    my $Subject = "$Options->{hostname}: Output from program $Options->{section}";

    if ($UseBinEnvPerl) {
        SendSmtpMail($Subject, "$TmpFileBase.mail", $Options, $Ini);
    } else {
        my %Args = ( ini => $Options->{ini} ? $Options->{ini} : $DefaultIni,
                     section => $Options->{section}, subject => $Subject,
            );
        if ($FileSize > 50000) { ## no critic qw( ProhibitMagicNumbers )
            $Args{attachment} = 1;
            debug(__LINE__, "SendTextMail: system(gzip -9 $TmpFileBase.run)");
            system("gzip -9 $TmpFileBase.run");
            $Args{file} = "$TmpFileBase.run.gz";
            $Args{descr} = MailHeader($Options, $Ini) if $Envelope & $MAIL_HEADER;
            $Args{descr} .= $ErrorMsg;
            $Args{descr} .= MailFooter() if $Envelope & $MAIL_FOOTER;
        } else {
            $Args{file} = "$TmpFileBase.mail";
        }
        my $Key = "MailSender:$Options->{hostname}";
        $Args{from} = $Ini->{$Options->{section}}{$Key};
        $Key = 'MailSender' unless defined $Args{from};
        $Args{from} = $Ini->{$Options->{section}}{$Key} unless defined $Args{from};
        debug(__LINE__, "Ini[$Options->{section}]$Key == $Args{from}");

        $Key = "MailRecipient:$Options->{hostname}";
        $Args{to} = $Ini->{$Options->{section}}{$Key};
        $Key = 'MailRecipient' unless defined $Args{to};
        $Args{to} = $Ini->{$Options->{section}}{$Key} unless defined $Args{to};
        debug(__LINE__, "Ini[$Options->{section}]$Key == $Args{to}") if $Args{to};

        $Key = "MailStore:$Options->{hostname}";
        $Args{store} = $Ini->{$Options->{section}}{$Key};
        $Key = 'MailStore' unless defined $Args{store};
        $Args{store} = $Ini->{$Options->{section}}{$Key} unless defined $Args{store};
        debug(__LINE__, "Ini[$Options->{section}]$Key == $Args{store}") if $Args{store};

        my $ArgString;
        foreach (qw(subject from to store file attachment)) {
            next unless $Args{$_};
            $ArgString .= " $_='$Args{$_}'";
        }
        debug(__LINE__, "SendTextMail:$ArgString");
        $FileTest = -s $Args{file} ? 1 : 0;
        debug(__LINE__, "SendTextMail: file == $Args{file} (not empty: $FileTest)");
        debug(undef, $Args{file});

        SendTextMail( \%Args ); ## no critic qw( ProhibitMissingImports )
    }
}

#========================================================================================
# function : ProcessInfo
#========================================================================================

sub ProcessInfo {
    return ($ENV{PERL}, (getpwuid($>))[0]) if $Config{osname} ne 'MSWin32';

    my $pi = Win32::Process::Info->new();
    foreach my $Proc (@{$pi->GetProcInfo()}) {
        next unless $Proc->{ProcessId} == $$;
        return ($^X, $Proc->{Owner});
    }
    return ($^X, undef);
}

#========================================================================================
# function : RunningProcessFound
#========================================================================================

sub RunningProcessFound {
    my ($PidFile, $Options, $Ini) = @_;

    my $ErrorMsg;
    my ($FileTest, $Error);
    if ( -e $PidFile ) {
        $FileTest = '(writeable: ';
        $FileTest .= -w $PidFile ? '1)' : '0)';
        $Error = -w $PidFile ? '' : '>>> ERROR <<<: ';
    } else {
        $FileTest = '(not found)';
    }
    debug(__LINE__, "${Error}check max duration ($Options->{maxage}): PidFile == $PidFile $FileTest");
    return 0 unless -e $PidFile;
    my $Pid;
    if (open my $PID_FILE, '<', $PidFile) {
        my @PidList = <$PID_FILE>;
        $Pid = $PidList[0];
        close $PID_FILE or $ErrorMsg = "Can't close $PidFile: $!";
    } else {
        $ErrorMsg = "Can't read $PidFile: $!\n";
    }

    if ($ErrorMsg) {
        ReportError($MAIL_HEADER, $ErrorMsg, $Options, $Ini);
        return 1;
    }

    $Pid =~ s/[^0-9]//sg;
    debug(__LINE__, "check max duration ($Options->{maxage}): PID == $Pid (PidFile: $PidFile)");
    return 0 unless $Pid;

    my $MaxAge;

    if ($Options->{maxage}) {
        my ($Minutes, $Hours, $Days);
        $Minutes = $Options->{maxage};
        if( $Minutes =~ s/(.*):// ) {
            $Hours = $1;
        }
        if( $Hours =~ s/(.*)-// ) {
            $Days = $1;
        }
        $Minutes = 0 unless $Minutes;
        $Hours = 0 unless $Hours;
        $Days = 0 unless $Days;
        $MaxAge = 60 * ($Days * 24 * 60 + $Hours * 60 + $Minutes); ## no critic qw(ProhibitMagicNumbers)
    } else {
        $MaxAge = 0;
    }

    my $LastProc;

    if ( $Config{osname} eq 'MSWin32' ) {

        my $pi =  Win32::Process::Info->new();
        foreach my $Proc (@{$pi->GetProcInfo()}) {
            if ($Proc->{ProcessId} == $Pid) {
                return 1 unless $MaxAge;
                return 1 if $CurrentTime - $Proc->{CreationDate} < $MaxAge;
                $LastProc = $Proc;
                last;
            }
        }
        if ($LastProc) {
            my $CreationDate = localtime($LastProc->{CreationDate});
            my $CurrentDate = localtime($CurrentTime);
            $ErrorMsg = <<EOF;
PidFile      : $PidFile
CSName       : $LastProc->{CSName}
CommandLine  : $LastProc->{CommandLine}
Owner        : $LastProc->{Owner}
CreationDate : $CreationDate
CurrentDate  : $CurrentDate
MaxDuration  : $Options->{maxage}
ProcessId    : $Pid
EOF
;
        } else {
            return 0;
        }

    } else {

        my $pi = Proc::ProcessTable->new;
        foreach my $Proc (@{$pi->table}) {
            if ($Proc->pid == $Pid) {
                return 1 unless $MaxAge;
                return 1 if $CurrentTime - $Proc->start < $MaxAge;
                $LastProc = $Proc;
                last;
            }
        }
        if ($LastProc) {
            my $CreationDate = localtime($LastProc->start);
            my $CurrentDate = localtime($CurrentTime);
            my $Owner = (getpwuid($LastProc->uid))[0];
            my $Group = (getgrgid($LastProc->gid))[0];
            my $CommandLine = $LastProc->cmndline;
            my $State = $LastProc->state;
            my $WorkingDir = $LastProc->cwd;
            $ErrorMsg = <<EOF;
PidFile      : $PidFile
CommandLine  : $CommandLine
State        : $State
WorkingDir   : $WorkingDir
Owner        : $Owner
Group        : $Group
CreationDate : $CreationDate
CurrentDate  : $CurrentDate
MaxDuration  : $Options->{maxage}
ProcessId    : $Pid
EOF
;
            debug(__LINE__, "RunningProcessFound: owner=$Owner, pid=$Pid, start=$CreationDate, cwd=$WorkingDir, state=$State, cmd=$CommandLine\n");
        } else {
            return 0;
        }
    }

    $ErrorMsg =  <<EOF;
Outdated scheduler ID file found.
Check the corresponding processes and stop them if necessary.
Delete the scheduler ID file, if the processes have stopped.

$ErrorMsg

EOF
;
    ReportError($MAIL_HEADER, $ErrorMsg, $Options, $Ini);
    return 1;
}

#========================================================================================
# function     RunPerlScript
#========================================================================================

sub RunPerlScript {
    my ($Options, $Ini) = @_;

    my $PidFile = VarFileName($Options, '.pid');
    my $TmpFileBase = TmpFileName($Options);

    my $ErrorMsg;

    if (open my $PID_FILE, '>', $PidFile) {
        print $PID_FILE $$;
        close $PID_FILE or $ErrorMsg = "Can't close $PidFile: $!\n";;
    } else {
        $ErrorMsg = "Can't write $PidFile: $!\n";
    }

    if ($ErrorMsg) {
        ReportError($MAIL_HEADER | $MAIL_FOOTER | $MAIL_FILE, $ErrorMsg, $Options, $Ini);
        return 1;
    }

    my ($PerlExe, undef) = ProcessInfo();
    my $Key = "Command:$Options->{hostname}";
    my $Script = $Ini->{$Options->{section}}{$Key};
    $Key = 'Command' unless $Script;
    $Script = $Ini->{$Options->{section}}{$Key} unless $Script;
    $Script = _ReplaceENV( $Script );
    my $Args = _ReplaceENV( $Ini->{$Options->{section}}{Options} );

    my $FileTest = -r $Script ? 1 : 0;
    my $Error = $FileTest ? '' : '>>> ERROR <<<: ';
    debug(__LINE__, "${Error}Ini[$Options->{section}]$Key == $Script (readable: $FileTest)");
    debug(__LINE__, "Ini[$Options->{section}]Options == $Args");

    unlink "$TmpFileBase.run";

    if ( $Config{osname} ne 'MSWin32' ) {
        debug(__LINE__, 'real/effective user : '. (getpwuid $<)[0]. '/'. (getpwuid $>)[0]);
        debug(__LINE__, 'real/effective group: '. (getgrgid $()[0]. '/'. (getgrgid $))[0]);
    }

    my ($Loop, $HasLoop);
    if ($Ini->{$Options->{section}}{Loop}) {
        $Loop =  $Ini->{$Options->{section}}{Loop};
        if ($Loop) {
            foreach my $LoopArgs (split(/,/, $Ini->{LOOPS}{$Loop})) {
                $HasLoop = 1;
                $LoopArgs = _ReplaceENV( $LoopArgs );
                debug(__LINE__, "system($PerlExe $Script $Args $LoopArgs >> \"$TmpFileBase.run\")");
                system("$PerlExe $Script $Args $LoopArgs >> \"$TmpFileBase.run\" 2>&1");
                $Error = $? == 0  ? '' : '>>> ERROR <<<: ';
                debug(__LINE__, "${Error}return code: $?");
                debug(undef, "$TmpFileBase.run");
            }
        }
    }
    debug(__LINE__, "system($PerlExe $Script $Args > \"$TmpFileBase.run\")") unless $HasLoop;
    system("$PerlExe $Script $Args > \"$TmpFileBase.run\" 2>&1") unless $HasLoop;
    $Error = $? == 0  ? '' : '>>> ERROR <<<: ';
    debug(__LINE__, "${Error}return code: $?");
    debug(undef, "$TmpFileBase.run");

    ReportError($MAIL_HEADER | $MAIL_FOOTER | $MAIL_FILE, undef, $Options, $Ini)
        if -s "$TmpFileBase.run";
    unlink $PidFile;
    return -s "$TmpFileBase.run";
}

# replace '$ENV{Varname}' in string by the actual values of environment variables
sub _ReplaceENV {
    my ($String) = @_;

    $String =~ s/\$ENV{(.*?)}/$ENV{$1}/ge;
    return $String;
}

#========================================================================================
# function : Main
#========================================================================================

sub Main {

    local $| = 1;

    my %Options;
    my @GetOpt = qw(ini=s var=s maxage=s help);

    GetOptions(\%Options, @GetOpt) or usage();
    usage() if $Options{help};

    usage() unless scalar @ARGV == 1;
    $Options{section} = $ARGV[0];

    debug(__LINE__, "$^X $0 $Options{section}");

    my $IniFile = $Options{ini} ? $Options{ini} : $DefaultIni;
    my $FileTest = -r $IniFile ? 1 : 0;
    my $Error = $FileTest ? '' : '>>> ERROR <<<: ';
    debug(__LINE__, "${Error}IniFile == $IniFile (readable: $FileTest)");
    usage() unless $FileTest;

    my %Ini;
    tie %Ini, 'Config::IniFiles', ( -file => $IniFile, -default => ':Default:' ) or die "$IniFile $!";

    if ($ENV{DEBUG_CRON}) {
        my $Dump = Dumper($Ini{$Options{section}});
        $Dump =~ s,^\$VAR1,[$Options{section}],;
        debug(undef, $Dump);
    }

    $Options{hostname} = hostname();
    chomp($Options{hostname});
    $Options{hostname} =~ s/\..*//;

    $Options{maxage} = $Ini{$Options{section}}{MaxDuration} unless $Options{maxage};
    $Options{isactive} = $Ini{$Options{section}}{IsActive};
    if ($Options{isactive} != 0) {
        $Options{isactive} = $Ini{'LOOPS'}{IsActive} if defined $Ini{'LOOPS'}{IsActive};
        $Options{isactive} = $Ini{':Default:'}{IsActive} if defined $Ini{':Default:'}{IsActive};
    }
    $Options{var} = $Ini{$Options{section}}{"VarDir:$Options{hostname}"} unless defined $Options{var};
    $Options{var} = $Ini{$Options{section}}{VarDir} unless defined $Options{var};
    $Options{var} = $ENV{EPAGES_LOG} unless $Options{var};

    debug(__LINE__, "isactive == $Options{isactive}");
    exit 0 if ($Options{isactive} == 0);

    $FileTest = -w $Options{var} ? 1 : 0;
    $Error = $FileTest ? '' : '>>> ERROR <<<: ';
    debug(__LINE__, "${Error}LogDir == $Options{var} (writeable: $FileTest)");

    $Options{tmp} = File::Temp->newdir();
    if (-d $Options{var}) {
        exit 0 if RunningProcessFound(VarFileName(\%Options, '.pid'), \%Options, \%Ini);
    } else {
        mkpath($Options{var});
    }

    exit RunPerlScript(\%Options, \%Ini);
}

if ($UseEpagesPerl) {
    RunScript( 'Sub' => \&Main );
} else {
    Main();
}

1;
